# Copyright 2017 - Nokia
# Copyright (c) 2014 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import re

from hacking import core

mutable_default_args = re.compile(r"^\s*def .+\((.+={\}|.+=\[\])")

asse_trueinst_re = re.compile(
    r"(.)*assertTrue\(isinstance\((\w|\.|\'|\"|\[|\])+, "
    r"(\w|\.|\'|\"|\[|\])+\)\)")
asse_equal_type_re = re.compile(
    r"(.)*assertEqual\(type\((\w|\.|\'|\"|\[|\])+\), "
    r"(\w|\.|\'|\"|\[|\])+\)")
asse_equal_end_with_none_re = re.compile(
    r"(.)*assertEqual\((\w|\.|\'|\"|\[|\])+, None\)")
asse_equal_start_with_none_re = re.compile(
    r"(.)*assertEqual\(None, (\w|\.|\'|\"|\[|\])+\)")
unicode_func_re = re.compile(r"(\s|\W|^)unicode\(")

_all_log_levels = {'debug', 'error', 'info', 'warning',
                   'critical', 'exception'}
# Since _Lx have been removed, we just need to check _()
translated_logs = re.compile(
    r"(.)*LOG\.(%(level)s)\(\s*_\(" % {'level': '|'.join(_all_log_levels)})

dict_constructor_with_list_copy_re = re.compile(r".*\bdict\((\[)?([(\[])")


@core.flake8ext
def assert_true_instance(logical_line):
    """Check for assertTrue(isinstance(a, b)) sentences

    V316
    """
    if asse_trueinst_re.match(logical_line):
        yield (0, "V316: assertTrue(isinstance(a, b)) sentences not allowed")


@core.flake8ext
def assert_equal_type(logical_line):
    """Check for assertEqual(type(A), B) sentences

    V317
    """
    if asse_equal_type_re.match(logical_line):
        yield (0, "V317: assertEqual(type(A), B) sentences not allowed")


@core.flake8ext
def no_translate_logs(logical_line):
    """Check for use of LOG.*(_(

    V319
    """
    if translated_logs.match(logical_line):
        yield (0, "V319: Don't translate logs")


@core.flake8ext
def check_no_contextlib_nested(logical_line):
    msg = ("V321: contextlib.nested is deprecated since Python 2.7. See "
           "https://docs.python.org/2/library/contextlib.html#contextlib."
           "nested for more information.")
    if ("with contextlib.nested(" in logical_line or
            "with nested(" in logical_line):
        yield(0, msg)


@core.flake8ext
def dict_constructor_with_list_copy(logical_line):
    msg = ("V322: Must use a dict comprehension instead of a dict constructor "
           "with a sequence of key-value pairs.")
    if dict_constructor_with_list_copy_re.match(logical_line):
        yield (0, msg)


@core.flake8ext
def check_python3_xrange(logical_line):
    if re.search(r"\bxrange\s*\(", logical_line):
        yield(0, "V323: Do not use xrange. Use range, or six.moves.range for "
                 "large loops.")


@core.flake8ext
def check_python3_no_iteritems(logical_line):
    msg = ("V324: Use six.iteritems() or dict.items() instead of "
           "dict.iteritems().")
    if re.search(r".*\.iteritems\(\)", logical_line):
        yield(0, msg)


@core.flake8ext
def check_python3_no_iterkeys(logical_line):
    msg = ("V325: Use six.iterkeys() or dict.keys() instead of "
           "dict.iterkeys().")
    if re.search(r".*\.iterkeys\(\)", logical_line):
        yield(0, msg)


@core.flake8ext
def check_python3_no_itervalues(logical_line):
    msg = ("V326: Use six.itervalues() or dict.values instead of "
           "dict.itervalues().")
    if re.search(r".*\.itervalues\(\)", logical_line):
        yield(0, msg)


@core.flake8ext
def no_mutable_default_args(logical_line):
    msg = "V327: Method's default argument shouldn't be mutable!"
    if mutable_default_args.match(logical_line):
        yield (0, msg)


@core.flake8ext
def no_log_warn(logical_line):
    """Disallow 'LOG.warn('

    V328
    """
    if logical_line.startswith('LOG.warn('):
        yield(0, 'V328: Use LOG.warning() rather than LOG.warn()')


@core.flake8ext
def check_assert_true_false(logical_line):
    """V329 - Don't use assertEqual(True/False, observed)."""
    if re.search(r"assertEqual\(\s*True,[^,]*(,[^,]*)?", logical_line):
        msg = ("V329: Use assertTrue(observed) instead of "
               "assertEqual(True, observed)")
        yield (0, msg)
    if re.search(r"assertEqual\([^,]*,\s*True(,[^,]*)?", logical_line):
        msg = ("V329: Use assertTrue(observed) instead of "
               "assertEqual(True, observed)")
        yield (0, msg)
    if re.search(r"assertEqual\(\s*False,[^,]*(,[^,]*)?", logical_line):
        msg = ("V329: Use assertFalse(observed) instead of "
               "assertEqual(False, observed)")
        yield (0, msg)
    if re.search(r"assertEqual\([^,]*,\s*False(,[^,]*)?", logical_line):
        msg = ("V329: Use assertFalse(observed) instead of "
               "assertEqual(False, observed)")
        yield (0, msg)



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from datetime import datetime
from oslo_config import cfg

from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.datasources.collectd import COLLECTD_DATASOURCE
from vitrage.datasources.collectd.driver import CollectdDriver
from vitrage.datasources.collectd.properties \
    import CollectdProperties as CProps
from vitrage.tests import base
from vitrage.tests.mocks import mock_driver


# noinspection PyProtectedMember
WARN_SEVERITY = 'warning'
WARNING_EVENT_TYPE = 'collectd.alarm.warning'
HOST = 'compute-1'


class TestCollectdDriver(base.BaseTest):
    OPTS = []

    # noinspection PyPep8Naming
    @classmethod
    def setUpClass(cls):
        super(TestCollectdDriver, cls).setUpClass()
        cls.conf = cfg.ConfigOpts()
        cls.conf.register_opts(cls.OPTS, group=COLLECTD_DATASOURCE)

    # noinspection PyAttributeOutsideInit
    def setUp(self):
        super(TestCollectdDriver, self).setUp()
        self.driver = CollectdDriver()

    def test_enrich_event_with_alarm_up(self):
        now = datetime.now().isoformat()

        event = self._enrich_event(now, HOST,
                                   WARN_SEVERITY,
                                   WARNING_EVENT_TYPE)

        self._assert_event_equal(event, WARNING_EVENT_TYPE,
                                 HOST, WARN_SEVERITY, now)

    def _enrich_event(self, time_now, hostname, severity, event_type):
        event = self._generate_event(time_now, hostname, severity)
        event = self.driver.enrich_event(event, event_type)
        return event

    @staticmethod
    def _generate_event(time, hostname, severity):
        update_vals = {}
        if hostname:
            update_vals[CProps.HOST] = hostname
        if severity:
            update_vals[CProps.SEVERITY] = severity

        if time:
            update_vals[CProps.TIME] = time

        generators = mock_driver.simple_doctor_alarm_generators(
            update_vals=update_vals)

        return mock_driver.generate_sequential_events_list(generators)[0]

    def _assert_event_equal(self,
                            event,
                            expected_event_type,
                            expected_hostname,
                            expected_severity,
                            expected_sample_date):
        self.assertIsNotNone(event, 'No event returned')
        self.assertEqual(expected_hostname, event[CProps.HOST])
        self.assertEqual(expected_severity, event[CProps.SEVERITY])
        self.assertEqual(expected_sample_date, event[CProps.TIME])
        self.assertEqual(expected_event_type, event[DSProps.EVENT_TYPE])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from pecan.core import abort

from vitrage.utils import recursive_keypairs


def enforce(rule, headers, enforcer, target):
    """Return the user and project the request should be limited to.

    :param headers: the request headers
    :param rule: The rule name
    :param enforcer: policy enforcer
    :param target: The target to enforce on.

    """
    creds = {
        'roles': headers.get('X-Roles', '').split(','),
        'user_id': headers.get('X-User-Id'),
        'project_id': headers.get('X-Project-Id'),
    }

    if not isinstance(target, dict):
        target = vars(target)

    target = dict(recursive_keypairs(target))

    if not enforcer.enforce(rule, target, creds):
        abort(status_code=403,
              detail='Authorization Failed')



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from vitrage.entity_graph.driver_exec import DriversNotificationEndpoint

from vitrage.datasources.driver_base import DriverBase
from vitrage.tests import base
from vitrage.tests.mocks import mock_driver


class MyTestDriver(DriverBase):
    def __init__(self):
        super(MyTestDriver, self).__init__()

    def enrich_event(self, event, event_type):
        return event

    def get_all(self, datasource_action):
        pass

    @staticmethod
    def get_event_types():
        pass


class TestListenerService(base.BaseTest):

    def _add_event_to_actual_events(self, events):
        self.actual_events.extend(events)

    def _set_excepted_events(self, events):
        self.excepted_events = events
        self.actual_events = []

    def _assert_events(self):
        self.assert_list_equal(self.excepted_events, self.actual_events)

    def _generate_events(self, update_events):
        gen_list = mock_driver.simple_aodh_alarm_notification_generators(
            1, update_events=update_events)
        events = mock_driver.generate_sequential_events_list(gen_list)
        self.excepted_events = events
        self.actual_events = []
        return events

    def test_notification_listener_endpoints(self):

        my_test_driver = MyTestDriver()
        enrich_callbacks_by_events = {"mock": [my_test_driver.enrich_event]}
        endpoint = DriversNotificationEndpoint(
            self._add_event_to_actual_events)
        endpoint._enrich_event_methods = enrich_callbacks_by_events

        # test handling one event
        events = self._generate_events(1)
        endpoint.info(None, None, "mock", events[0], None)
        self._assert_events()

        # test handling list of events
        events = self._generate_events(2)
        endpoint.info(None, None, "mock", events, None)
        self._assert_events()



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from vitrage.common.constants import EntityCategory
from vitrage.evaluator.template_fields import TemplateFields
from vitrage.evaluator.template_validation.status_messages import status_msgs
from vitrage.tests import base


DEFINITIONS_INDEX_MOCK = {
    '123': {},
    '456': {},
    '789': {},
    'a1': {
        TemplateFields.CATEGORY: EntityCategory.ALARM
    },
    'a2': {
        TemplateFields.CATEGORY: EntityCategory.ALARM
    }
}


class ValidatorTest(base.BaseTest):

    def _assert_correct_result(self, result):

        self.assertTrue(result.is_valid_config)
        self.assertEqual(result.comment, status_msgs[0])
        self.assertEqual(0, result.status_code)

    def _assert_fault_result(self, result, status_code):

        self.assertFalse(result.is_valid_config)
        self.assertTrue(result.comment.startswith(status_msgs[status_code]))
        self.assertEqual(result.status_code, status_code)

    def _assert_warning_result(self, result, status_code):

        self.assertTrue(result.is_valid_config)
        self.assertTrue(result.comment.startswith(status_msgs[status_code]))
        self.assertEqual(result.status_code, status_code)

    @staticmethod
    def _hide_useless_logging_messages():

        validator_path = 'vitrage.evaluator.template_validation.' \
                         'template_content_validator'
        content_validator_log = logging.getLogger(validator_path)
        content_validator_log.setLevel(logging.FATAL)


class ActionValidatorTest(ValidatorTest):

    # noinspection PyPep8Naming
    @classmethod
    def setUpClass(cls):
        super(ActionValidatorTest, cls).setUpClass()

        cls._hide_useless_logging_messages()

    def _validate_action(self, action, validation_func):

        # Test setup
        idx = DEFINITIONS_INDEX_MOCK.copy()

        # Test action and assertions
        result = validation_func(action, idx)

        # Test Assertions
        self._assert_correct_result(result)

    def _validate_action_without_action_target(self, action, validation_func):

        # Test setup
        idx = DEFINITIONS_INDEX_MOCK.copy()
        action.pop(TemplateFields.ACTION_TARGET)

        # Test action
        result = validation_func(action, idx)

        # Test assertions
        self._assert_fault_result(result, 124)

    def _validate_action_with_invalid_target_id(self, action, validation_func):

        # Test setup
        idx = DEFINITIONS_INDEX_MOCK.copy()

        # Test action
        result = validation_func(action, idx)

        # Test assertions
        self._assert_fault_result(result, 3)

    def _validate_action_without_target_id(self,
                                           action,
                                           validation_func,
                                           expected_status_code):

        # Test setup
        idx = DEFINITIONS_INDEX_MOCK.copy()
        action[TemplateFields.ACTION_TARGET].pop(TemplateFields.TARGET)

        # Test action
        result = validation_func(action, idx)

        # Test assertions
        self._assert_fault_result(result, expected_status_code)



# Copyright 2016 - Alcatel-Lucent
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_vitrage graph
----------------------------------

Tests for `vitrage` graph driver
"""

from oslo_log import log as logging

from testtools import matchers

from vitrage.common.constants import EdgeProperties as EProps
from vitrage.common.constants import EntityCategory
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources import NOVA_HOST_DATASOURCE
from vitrage.datasources import NOVA_INSTANCE_DATASOURCE
from vitrage.datasources import OPENSTACK_CLUSTER
from vitrage.graph import Direction
from vitrage.graph.filter import check_filter
from vitrage.graph import utils

from vitrage.tests import base
from vitrage.tests.base import IsEmpty
from vitrage.tests.unit.graph.base import ALARM
from vitrage.tests.unit.graph.base import ALARM_ON_HOST
from vitrage.tests.unit.graph.base import ALARM_ON_VM
from vitrage.tests.unit.graph.base import e_node_to_host
from vitrage.tests.unit.graph.base import GraphTestBase
from vitrage.tests.unit.graph.base import NXGraph
from vitrage.tests.unit.graph.base import RESOURCE
from vitrage.tests.unit.graph.base import v_alarm
from vitrage.tests.unit.graph.base import v_host
from vitrage.tests.unit.graph.base import v_instance
from vitrage.tests.unit.graph.base import v_node

LOG = logging.getLogger(__name__)


class TestGraph(GraphTestBase):

    def test_graph(self):
        g = NXGraph('test_graph')
        self.assertEqual('test_graph', g.name, 'graph name')
        self.assertThat(g, IsEmpty(), 'graph __len__')

        g.add_vertex(v_node)
        g.add_vertex(v_host)
        g.add_edge(e_node_to_host)
        self.assertThat(g,
                        matchers.HasLength(2),
                        'graph __len__ after add vertices')

        graph_copy = g.copy()
        self.assertEqual('test_graph', graph_copy.name, 'graph copy name')
        self.assertThat(graph_copy,
                        matchers.HasLength(2),
                        'graph copy __len__')

        g.remove_vertex(v_node)
        self.assertThat(g, matchers.HasLength(1),
                        'graph __len__ after remove vertex')
        self.assertThat(graph_copy, matchers.HasLength(2),
                        'graph copy __len__')

        updated_vertex = g.get_vertex(v_host.vertex_id)
        updated_vertex[VProps.VITRAGE_CATEGORY] = ALARM
        g.update_vertex(updated_vertex)
        v_from_g = g.get_vertex(v_host.vertex_id)
        v_from_graph_copy = graph_copy.get_vertex(v_host.vertex_id)
        self.assertEqual(ALARM, v_from_g[VProps.VITRAGE_CATEGORY],
                         'graph vertex changed after update')
        self.assertEqual(NOVA_HOST_DATASOURCE,
                         v_from_graph_copy[VProps.VITRAGE_TYPE],
                         'graph copy vertex unchanged after update')

    def test_vertex_crud(self):
        g = NXGraph('test_vertex_crud')
        g.add_vertex(v_node)
        v = g.get_vertex(v_node.vertex_id)
        self.assertEqual(v_node[VProps.ID], v[VProps.ID],
                         'vertex properties are saved')
        self.assertEqual(v_node[VProps.VITRAGE_CATEGORY],
                         v[VProps.VITRAGE_CATEGORY],
                         'vertex properties are saved')
        self.assertEqual(v_node.vertex_id, v.vertex_id,
                         'vertex vertex_id is saved')

        # Changing the referenced item
        updated_v = v
        updated_v['KUKU'] = 'KUKU'
        updated_v[VProps.VITRAGE_CATEGORY] = 'CHANGED'
        # Get it again
        v = g.get_vertex(v_node.vertex_id)
        self.assertIsNone(v.get('KUKU', None),
                          'Change should not affect graph item')
        self.assertFalse(v.get(EProps.VITRAGE_IS_DELETED, None),
                         'Change should not affect graph item')
        self.assertEqual(v_node[VProps.VITRAGE_CATEGORY],
                         v[VProps.VITRAGE_CATEGORY],
                         'Change should not affect graph item')
        # Update the graph item and see changes take place
        g.update_vertex(updated_v)
        # Get it again
        v = g.get_vertex(v_node.vertex_id)
        self.assertEqual(updated_v['KUKU'], v['KUKU'],
                         'Graph item should change after update')
        self.assertEqual(updated_v[VProps.VITRAGE_CATEGORY],
                         v[VProps.VITRAGE_CATEGORY],
                         'Graph item should change after update')

        # Update the graph item and see changes take place
        updated_v['KUKU'] = None
        g.update_vertex(updated_v)
        # Get it again
        v = g.get_vertex(v_node.vertex_id)
        self.assertNotIn('KUKU', v.properties,
                         'Update value to None should entirely remove the key')

        # check metadata
        another_vertex = utils.create_vertex(
            vitrage_id='123',
            vitrage_category=NOVA_INSTANCE_DATASOURCE,
            entity_id='456',
            metadata={'some_meta': 'DATA',
                      VProps.VITRAGE_TYPE: NOVA_INSTANCE_DATASOURCE,
                      VProps.RESOURCE_ID: 'sdg7849ythksjdg'}
        )
        g.add_vertex(another_vertex)
        v = g.get_vertex(another_vertex.vertex_id)
        self.assertEqual(another_vertex[VProps.ID], v[VProps.ID],
                         'vertex properties are saved')
        self.assertEqual(another_vertex[VProps.VITRAGE_CATEGORY],
                         v[VProps.VITRAGE_CATEGORY],
                         'vertex properties are saved')
        self.assertEqual('DATA', v['some_meta'],
                         'vertex properties are saved')
        self.assertEqual(another_vertex.vertex_id, v.vertex_id,
                         'vertex vertex_id is saved')

        # Remove the item
        g.remove_vertex(another_vertex)
        self.assertThat(g, matchers.HasLength(1),
                        'graph __len__ after remove vertex')
        v = g.get_vertex(another_vertex.vertex_id)
        self.assertIsNone(v, 'removed vertex not in graph')

    def test_update_vertices(self):

        # Test Setup
        g = NXGraph('test_update_vertices')
        g.add_vertex(v_node)
        v_node_copy = g.get_vertex(v_node.vertex_id)
        v_node_copy[VProps.NAME] = 'test_node'
        v_node_copy[VProps.VITRAGE_CATEGORY] = 'super_node'

        g.add_vertex(v_host)
        v_host_copy = g.get_vertex(v_host.vertex_id)
        v_host_copy[VProps.NAME] = 'test_host'

        # Test Action
        g.update_vertices([v_node_copy, v_host_copy])

        # Test Assertions
        updated_v_node = g.get_vertex(v_node.vertex_id)
        self.assertEqual('test_node', updated_v_node[VProps.NAME])
        self.assertEqual('super_node', updated_v_node[VProps.VITRAGE_CATEGORY])

        updated_v_host = g.get_vertex(v_host.vertex_id)
        self.assertEqual('test_host', updated_v_host[VProps.NAME])

    def test_edge_crud(self):
        g = NXGraph('test_edge_crud')
        g.add_vertex(v_node)
        g.add_vertex(v_host)
        g.add_edge(e_node_to_host)
        self.assertEqual(1, g.num_edges(), 'graph __len__ after add edge')
        label = e_node_to_host[EProps.RELATIONSHIP_TYPE]
        e = g.get_edge(v_node.vertex_id, v_host.vertex_id, label)
        self.assertEqual(e_node_to_host[EProps.RELATIONSHIP_TYPE],
                         e[EProps.RELATIONSHIP_TYPE],
                         'edge properties are saved')
        self.assertEqual(e_node_to_host.source_id, e.source_id,
                         'edge vertex_id is saved')
        self.assertEqual(e_node_to_host.target_id, e.target_id,
                         'edge vertex_id is saved')

        # Edge is correct
        v_node_neig = g.neighbors(v_node.vertex_id, direction=Direction.OUT)
        self.assertThat(v_node_neig, matchers.HasLength(1),
                        'v_node OUT neighbor count')
        self.assertEqual(v_host.vertex_id, v_node_neig.pop().vertex_id,
                         'v_node OUT neighbor is v_host')
        v_node_neig = g.neighbors(v_node.vertex_id, direction=Direction.IN)
        self.assertThat(v_node_neig, IsEmpty(), 'v_node IN neighbor count')
        v_host_neig = g.neighbors(v_host.vertex_id, direction=Direction.OUT)
        self.assertThat(v_host_neig, IsEmpty(), 'v_host OUT neighbor count')
        v_host_neig = g.neighbors(v_host.vertex_id, direction=Direction.IN)
        self.assertThat(v_host_neig, matchers.HasLength(1),
                        'v_host IN neighbor count')
        self.assertEqual(v_node.vertex_id, v_host_neig.pop().vertex_id,
                         'v_host IN neighbor is v_node')

        # Changing the referenced item
        updated_e = e
        updated_e[EProps.VITRAGE_IS_DELETED] = 'KUKU'
        updated_e[EProps.UPDATE_TIMESTAMP] = 'CHANGED'

        # Get it again
        e = g.get_edge(v_node.vertex_id, v_host.vertex_id, label)
        self.assertFalse(e.get(EProps.VITRAGE_IS_DELETED, None),
                         'Change should not affect graph item')
        self.assertEqual(e_node_to_host[EProps.UPDATE_TIMESTAMP],
                         e[EProps.UPDATE_TIMESTAMP],
                         'Change should not affect graph item')
        # Update the graph item and see changes take place
        g.update_edge(updated_e)
        # Get it again
        e = g.get_edge(v_node.vertex_id, v_host.vertex_id, label)
        self.assertEqual(updated_e[EProps.VITRAGE_IS_DELETED],
                         e[EProps.VITRAGE_IS_DELETED],
                         'Graph item should change after update')
        self.assertEqual(updated_e[EProps.UPDATE_TIMESTAMP],
                         e[EProps.UPDATE_TIMESTAMP],
                         'Graph item should change after update')

        # Update the graph item and see changes take place
        updated_e[EProps.VITRAGE_IS_DELETED] = None
        g.update_edge(updated_e)
        # Get it again
        e = g.get_edge(v_node.vertex_id, v_host.vertex_id, label)
        self.assertNotIn(EProps.VITRAGE_IS_DELETED, e.properties,
                         'Update value to None should entirely remove the key')

        # check metadata
        another_label = 'ANOTHER_LABEL'
        another_edge = utils.create_edge(
            source_id=v_node.vertex_id,
            target_id=v_host.vertex_id,
            relationship_type=another_label,
            metadata={'some_meta': 'DATA'})
        g.add_edge(another_edge)
        self.assertEqual(2, g.num_edges(), 'graph __len__ after add edge')
        e = g.get_edge(v_node.vertex_id, v_host.vertex_id, another_label)
        self.assertEqual(another_edge[EProps.RELATIONSHIP_TYPE],
                         e[EProps.RELATIONSHIP_TYPE],
                         'edge properties are saved')
        self.assertEqual('DATA', e['some_meta'],
                         'edge properties are saved')

        # Remove the item
        g.remove_edge(another_edge)
        self.assertEqual(1, g.num_edges(), 'graph __len__ after remove edge')
        e = g.get_edge(v_node.vertex_id, v_host.vertex_id, another_label)
        self.assertIsNone(e, 'removed edge not in graph')

        # Check get_edge returns None when item is missing
        edge = g.get_edge(v_host.vertex_id, 'ddd', '333')
        self.assertIsNone(edge)
        edge = g.get_edge('eee', v_node.vertex_id, '333')
        self.assertIsNone(edge)
        edge = g.get_edge(v_host.vertex_id, v_node.vertex_id, None)
        self.assertIsNone(edge)
        edge = g.get_edge(None, v_node.vertex_id, '333')
        self.assertIsNone(edge)

    def test_neighbors(self):
        relationship_a = 'RELATIONSHIP_A'
        relationship_b = 'RELATIONSHIP_B'
        relationship_c = 'RELATIONSHIP_C'

        v1 = v_node
        v2 = v_host
        v3 = v_instance
        v4 = v_alarm
        v5 = utils.create_vertex(
            vitrage_id='kuku',
            vitrage_category=EntityCategory.RESOURCE,
            vitrage_type=NOVA_HOST_DATASOURCE)

        g = NXGraph('test_neighbors')
        g.add_vertex(v1)
        g.add_vertex(v2)
        g.add_vertex(v3)
        g.add_vertex(v4)
        g.add_vertex(v5)

        g.add_edge(utils.create_edge(source_id=v1.vertex_id,
                                     target_id=v2.vertex_id,
                                     relationship_type=relationship_a))
        g.add_edge(utils.create_edge(source_id=v1.vertex_id,
                                     target_id=v2.vertex_id,
                                     relationship_type=relationship_b))
        g.add_edge(utils.create_edge(source_id=v1.vertex_id,
                                     target_id=v4.vertex_id,
                                     relationship_type=relationship_a))
        g.add_edge(utils.create_edge(source_id=v1.vertex_id,
                                     target_id=v4.vertex_id,
                                     relationship_type=relationship_b))
        g.add_edge(utils.create_edge(source_id=v2.vertex_id,
                                     target_id=v1.vertex_id,
                                     relationship_type=relationship_c))
        g.add_edge(utils.create_edge(source_id=v2.vertex_id,
                                     target_id=v3.vertex_id,
                                     relationship_type=relationship_a))
        g.add_edge(utils.create_edge(source_id=v2.vertex_id,
                                     target_id=v3.vertex_id,
                                     relationship_type=relationship_b))
        g.add_edge(utils.create_edge(source_id=v2.vertex_id,
                                     target_id=v4.vertex_id,
                                     relationship_type=relationship_a))
        g.add_edge(utils.create_edge(source_id=v4.vertex_id,
                                     target_id=v1.vertex_id,
                                     relationship_type=relationship_c))

        # CHECK V1

        v1_neighbors = g.neighbors(v_id=v1.vertex_id)
        self._assert_set_equal({v2, v4}, v1_neighbors, 'Check V1 neighbors')

        v1_neighbors = g.neighbors(
            v_id=v1.vertex_id,
            vertex_attr_filter={VProps.VITRAGE_TYPE: NOVA_HOST_DATASOURCE})
        self._assert_set_equal({v2}, v1_neighbors,
                               'Check V1 neighbors, vertex property filter')

        v1_neighbors = g.neighbors(
            v_id=v1.vertex_id,
            edge_attr_filter={EProps.RELATIONSHIP_TYPE: relationship_a})
        self._assert_set_equal({v2, v4}, v1_neighbors,
                               'Check V1 neighbors, edge property filter')

        v1_neighbors = g.neighbors(v_id=v1.vertex_id,
                                   direction=Direction.IN)
        self._assert_set_equal({v2, v4}, v1_neighbors,
                               'Check V1 neighbors, direction IN')

        v1_neighbors = g.neighbors(v_id=v1.vertex_id,
                                   direction=Direction.OUT)
        self._assert_set_equal({v2, v4}, v1_neighbors,
                               'Check V1 neighbors, direction OUT')

        v1_neighbors = g.neighbors(v_id=v1.vertex_id,
                                   direction=Direction.BOTH)
        self._assert_set_equal({v2, v4}, v1_neighbors,
                               'Check V1 neighbors, direction BOTH')

        v1_neighbors = g.neighbors(
            v_id=v1.vertex_id,
            direction=Direction.IN,
            edge_attr_filter={EProps.RELATIONSHIP_TYPE: relationship_c},
            vertex_attr_filter={VProps.VITRAGE_TYPE: NOVA_HOST_DATASOURCE})
        self._assert_set_equal(
            {v2}, v1_neighbors,
            'Check V1 neighbors, vertex/edge property filter and direction')

        # CHECK V2

        v2_neighbors = g.neighbors(v_id=v2.vertex_id)
        self._assert_set_equal({v1, v3, v4}, v2_neighbors,
                               'Check v2 neighbors')

        v2_neighbors = g.neighbors(
            v_id=v2.vertex_id,
            vertex_attr_filter={VProps.VITRAGE_CATEGORY: NOVA_HOST_DATASOURCE})
        self._assert_set_equal({}, v2_neighbors,
                               'Check v2 neighbors, vertex property filter')

        v2_neighbors = g.neighbors(
            v_id=v2.vertex_id,
            vertex_attr_filter={VProps.VITRAGE_CATEGORY: [NOVA_HOST_DATASOURCE,
                                                          ALARM]})
        self._assert_set_equal({v4}, v2_neighbors,
                               'Check v2 neighbors, vertex property filter')

        v2_neighbors = g.neighbors(
            v_id=v2.vertex_id,
            edge_attr_filter={
                EProps.RELATIONSHIP_TYPE: [relationship_a, relationship_b]
            },
            vertex_attr_filter={
                VProps.VITRAGE_CATEGORY: [RESOURCE, ALARM],
                VProps.VITRAGE_TYPE: [NOVA_HOST_DATASOURCE,
                                      NOVA_INSTANCE_DATASOURCE,
                                      ALARM_ON_VM,
                                      ALARM_ON_HOST]
            }
        )
        self._assert_set_equal({v3, v4}, v2_neighbors,
                               'Check v2 neighbors, edge property filter')

        # CHECK V3

        v3_neighbors = g.neighbors(v_id=v3.vertex_id, direction=Direction.OUT)
        self._assert_set_equal({}, v3_neighbors,
                               'Check v3 neighbors, direction OUT')

        v3_neighbors = g.neighbors(
            v_id=v3.vertex_id,
            vertex_attr_filter={VProps.VITRAGE_CATEGORY: NOVA_HOST_DATASOURCE},
            direction=Direction.OUT)
        self._assert_set_equal({}, v3_neighbors,
                               'Check neighbors for vertex without any')
        v5_neighbors = g.neighbors(
            v_id=v5.vertex_id,
            vertex_attr_filter={VProps.VITRAGE_CATEGORY: NOVA_HOST_DATASOURCE})
        self._assert_set_equal({}, v5_neighbors,
                               'Check neighbors for not connected vertex')

    def test_get_vertices(self):
        g = NXGraph('test_get_vertices')
        g.add_vertex(v_node)
        g.add_vertex(v_host)
        g.add_edge(e_node_to_host)

        all_vertices = g.get_vertices()
        self.assertThat(all_vertices, matchers.HasLength(2),
                        'get_vertices __len__ all vertices')

        node_vertices = g.get_vertices(
            vertex_attr_filter={VProps.VITRAGE_TYPE: OPENSTACK_CLUSTER})
        self.assertThat(node_vertices, matchers.HasLength(1),
                        'get_vertices __len__ node vertices')
        found_vertex = node_vertices.pop()
        self.assertEqual(OPENSTACK_CLUSTER, found_vertex[VProps.VITRAGE_TYPE],
                         'get_vertices check node vertex')

        node_vertices = g.get_vertices(
            vertex_attr_filter={VProps.VITRAGE_TYPE: OPENSTACK_CLUSTER,
                                VProps.VITRAGE_CATEGORY: RESOURCE})
        self.assertThat(node_vertices, matchers.HasLength(1),
                        'get_vertices __len__ node vertices')
        found_vertex = node_vertices.pop()
        self.assertEqual(OPENSTACK_CLUSTER, found_vertex[VProps.VITRAGE_TYPE],
                         'get_vertices check node vertex')

    def _check_callbacks_result(self, msg, exp_prev, exp_curr):

        def assert_none_or_equals(exp, act, message):
            if exp:
                self.assertEqual(exp, act, message)
            else:
                self.assertIsNone(act, message)

        self.assertIsNotNone(self.result, msg + ' Callback was not called')
        assert_none_or_equals(exp_prev, self.result[0],
                              msg + ' prev_item unexpected')
        assert_none_or_equals(exp_curr, self.result[1],
                              msg + ' curr_item unexpected')

        self.assertEqual(self.result, self.final_result,
                         'callback order is incorrect')

        self.result = None
        self.final_result = None

    def _assert_none_or_equals(self, exp, act, msg):
        if exp:
            self.assertEqual(exp, act, msg)
        else:
            self.assertIsNone(act, msg)

    # noinspection PyAttributeOutsideInit
    def test_graph_callbacks(self):

        g = NXGraph('test_graph_callbacks')
        self.result = None
        self.final_result = None

        def callback_2(pre_item, current_item, is_vertex, graph):
            # We want to make sure this callback was called ^after^ the other
            # And expect that the later callback copies the result from the
            # prior call, hence these should be equal after both were called
            self.final_result = self.result

        def callback(pre_item, current_item, is_vertex, graph):
            LOG.info('called with: pre_event_item ' + str(pre_item) +
                     ' current_item ' + str(current_item))
            self.assertIsNotNone(current_item)
            self.result = pre_item, current_item, is_vertex

        # Check there is no notification without subscribing
        g.add_vertex(v_alarm)
        self.assertIsNone(
            self.result,
            'Got notification, but add_vertex notification is not registered')

        # subscribe
        g.subscribe(callback_2, finalization=True)
        g.subscribe(callback)

        # These actions will trigger callbacks:
        g.add_vertex(v_node)
        self._check_callbacks_result('add vertex', None, v_node)

        g.add_vertex(v_host)
        self._check_callbacks_result('add vertex', None, v_host)

        g.add_edge(e_node_to_host)
        self._check_callbacks_result('add edge', None, e_node_to_host)

        updated_vertex = g.get_vertex(v_host.vertex_id)
        updated_vertex[VProps.VITRAGE_CATEGORY] = ALARM
        g.update_vertex(updated_vertex)
        self._check_callbacks_result('update vertex', v_host, updated_vertex)

        updated_edge = g.get_edge(e_node_to_host.source_id,
                                  e_node_to_host.target_id,
                                  e_node_to_host.label)
        updated_edge['ZIG'] = 'ZAG'
        g.update_edge(updated_edge)
        self._check_callbacks_result('update edge', e_node_to_host,
                                     updated_edge)

    def test_union(self):
        v1 = v_node
        v2 = v_host
        v3 = v_instance
        v4 = v_alarm

        e_v1_v2 = utils.create_edge(source_id=v1.vertex_id,
                                    target_id=v2.vertex_id,
                                    relationship_type='KUKU_v1_v2')
        e_v2_v3 = utils.create_edge(source_id=v2.vertex_id,
                                    target_id=v3.vertex_id,
                                    relationship_type='KUKU_v2_v3')
        e_v3_v4 = utils.create_edge(source_id=v3.vertex_id,
                                    target_id=v4.vertex_id,
                                    relationship_type='KUKU_v3_v4')

        g1 = NXGraph('test_union')
        g1.add_vertex(v1)
        g1.add_vertex(v2)
        g1.add_vertex(v3)
        g1.add_edge(e_v1_v2)
        g1.add_edge(e_v2_v3)

        g2 = NXGraph('test_union_')
        g2.add_vertex(v3)
        g2.add_vertex(v4)
        g2.add_edge(e_v3_v4)

        g1.union(g2)
        self.assertThat(g1, matchers.HasLength(4),
                        'incorrect graph len after union')

        e = g1.get_edge(e_v3_v4.source_id, e_v3_v4.target_id, e_v3_v4.label)
        self.assertIsNotNone(e, 'Edge missing after graphs union')

        e = g1.get_edge(e_v2_v3.source_id, e_v2_v3.target_id, e_v2_v3.label)
        self.assertIsNotNone(e, 'Edge missing after graphs union')

        e = g1.get_vertex(v3.vertex_id)
        self.assertIsNotNone(e, 'Vertex missing after graphs union')


class TestFilter(base.BaseTest):

    def test_basic_regex(self):
        event_properties = {
            "time": 121354,
            "vitrage_type": "zabbix",
            "vitrage_category": "ALARM",
            "rawtext": "Interface kukoo down on {HOST.NAME}",
            "host": "some_host_kukoo"
        }

        attr_filter = {
            "vitrage_category": "ALARM",
            "rawtext.regex": r"Interface ([_a-zA-Z0-9'\-]+) down on {"
                             "HOST.NAME}",
            "host": "some_host_kukoo"
        }
        self.assertTrue(check_filter(data=event_properties,
                                     attr_filter=attr_filter))

    def test_basic_regex_with_no_match(self):
        event_properties = {
            "time": 121354,
            "vitrage_type": "zabbix",
            "vitrage_category": "ALARM",
            "rawtext": "Text With No Match",
            "host": "some_host_kukoo"
        }

        attr_filter = {
            "vitrage_category": "ALARM",
            "rawtext.RegEx": r"Interface ([_a-zA-Z0-9'\-]+) down on {"
                             "HOST.NAME}",
            "host": "some_host_kukoo"
        }
        self.assertFalse(check_filter(data=event_properties,
                                      attr_filter=attr_filter))



# Copyright 2018 Samsung Electronics
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime

from oslo_config import cfg
from oslo_log import log as logging
from testtools import matchers

from vitrage.common.constants import DatasourceOpts as DSOpts
from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import EdgeLabel
from vitrage.common.constants import EntityCategory
from vitrage.common.constants import UpdateMethod
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources import transformer_base as tb
from vitrage.datasources.transformer_base import TransformerBase
from vitrage.datasources.trove.cluster.transformer import \
    TroveClusterTransformer
from vitrage.datasources.trove.cluster import TROVE_CLUSTER_DATASOURCE
from vitrage.datasources.trove.instance import TROVE_INSTANCE_DATASOURCE
from vitrage.tests import base
from vitrage.tests.mocks import mock_driver as mock_sync

LOG = logging.getLogger(__name__)


class TroveClusterTransformerTest(base.BaseTest):

    OPTS = [
        cfg.StrOpt(DSOpts.UPDATE_METHOD, default=UpdateMethod.PULL),
    ]

    @classmethod
    def setUpClass(cls):
        super(TroveClusterTransformerTest, cls).setUpClass()
        cls.transformers = {}
        cls.conf = cfg.ConfigOpts()
        cls.conf.register_opts(cls.OPTS, group=TROVE_CLUSTER_DATASOURCE)
        cls.transformers[TROVE_CLUSTER_DATASOURCE] = \
            TroveClusterTransformer(cls.transformers)

    def test_create_placeholder_vertex(self):
        # Tests setup
        cluster_id = 'tr-cluster-0'
        timestamp = datetime.datetime.utcnow()

        properties = {
            VProps.ID: cluster_id,
            VProps.VITRAGE_TYPE: TROVE_CLUSTER_DATASOURCE,
            VProps.VITRAGE_CATEGORY: EntityCategory.RESOURCE,
            VProps.VITRAGE_SAMPLE_TIMESTAMP: timestamp
        }

        transformer = self.transformers[TROVE_CLUSTER_DATASOURCE]

        # Test action
        placeholder = transformer.create_neighbor_placeholder_vertex(
            **properties)

        # Test assertions
        expected_key = tb.build_key(
            transformer._key_values(TROVE_CLUSTER_DATASOURCE, cluster_id))
        expected_uuid = TransformerBase.uuid_from_deprecated_vitrage_id(
            expected_key)
        self.assertEqual(expected_uuid, placeholder.vertex_id)

        self.assertEqual(timestamp,
                         placeholder.get(VProps.VITRAGE_SAMPLE_TIMESTAMP))

        self.assertEqual(TROVE_CLUSTER_DATASOURCE,
                         placeholder.get(VProps.VITRAGE_TYPE))

        self.assertEqual(cluster_id, placeholder.get(VProps.ID))

        self.assertEqual(EntityCategory.RESOURCE,
                         placeholder.get(VProps.VITRAGE_CATEGORY))

        self.assertTrue(placeholder.get(VProps.VITRAGE_IS_PLACEHOLDER))

    def test_snapshot_event_transform(self):
        # Test setup
        spec_list = mock_sync.simple_trove_cluster_generators(
            clust_num=1, inst_num=1, snapshot_events=10)
        events = mock_sync.generate_random_events_list(spec_list)

        for event in events:
            # Test action
            transformer = self.transformers[TROVE_CLUSTER_DATASOURCE]
            wrapper = transformer.transform(event)

            # Test assertions
            vertex = wrapper.vertex
            self._validate_vertex_props(vertex, event)

            neighbours = wrapper.neighbors
            self.assertThat(neighbours, matchers.HasLength(1))
            self._validate_server_neighbour(neighbours[0], vertex.vertex_id,
                                            event)

    def _validate_vertex_props(self, vertex, event):
        self.assertEqual(event['id'], vertex[VProps.ID])

        self.assertEqual(event['name'], vertex[VProps.NAME])

        self.assertEqual(event['task']['name'], vertex[VProps.STATE])

        self.assertEqual(event[DSProps.SAMPLE_DATE],
                         vertex[VProps.VITRAGE_SAMPLE_TIMESTAMP])

        self.assertEqual(EntityCategory.RESOURCE,
                         vertex[VProps.VITRAGE_CATEGORY])

        self.assertEqual(TROVE_CLUSTER_DATASOURCE,
                         vertex[VProps.VITRAGE_TYPE])

        self.assertFalse(vertex[VProps.VITRAGE_IS_PLACEHOLDER])
        self.assertFalse(vertex[VProps.VITRAGE_IS_DELETED])

    def _validate_server_neighbour(self, neighbour, cluster_id, event):
        vertex, edge = neighbour.vertex, neighbour.edge

        # Validate neighbor vertex
        self.assertEqual(EntityCategory.RESOURCE,
                         vertex[VProps.VITRAGE_CATEGORY])

        self.assertEqual(TROVE_INSTANCE_DATASOURCE,
                         vertex[VProps.VITRAGE_TYPE])

        instance_id = event['instances'][0]['id']
        self.assertEqual(instance_id, vertex[VProps.ID])

        self.assertTrue(vertex[VProps.VITRAGE_IS_PLACEHOLDER])
        self.assertFalse(vertex[VProps.VITRAGE_IS_DELETED])

        # Validate neighbor edge
        self.assertEqual(edge.target_id, vertex.vertex_id)
        self.assertEqual(edge.source_id, cluster_id)
        self.assertEqual(edge.label, EdgeLabel.CONTAINS)



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg
from oslo_log import log

from vitrage.common.constants import DatasourceAction
from vitrage.common.constants import DatasourceOpts as DSOpts
from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.datasources.alarm_driver_base import AlarmDriverBase
from vitrage.datasources.collectd import COLLECTD_DATASOURCE
from vitrage.datasources.collectd.mapper import CollectdMapper
from vitrage.datasources.collectd.properties\
    import CollectdProperties as CProps
from vitrage.utils import file as file_utils

CONF = cfg.CONF
LOG = log.getLogger(__name__)


class CollectdDriver(AlarmDriverBase):
    conf_map = None

    def __init__(self):
        super(CollectdDriver, self).__init__()
        if not CollectdDriver.conf_map:
            mapper = CollectdDriver._configuration_mapping()
            if mapper:
                CollectdDriver.conf_map = CollectdMapper(mapper)

    def _vitrage_type(self):
        return COLLECTD_DATASOURCE

    def _alarm_key(self, alarm):
        return alarm[CProps.ID]

    def _get_alarms(self):
        return []

    def _is_erroneous(self, alarm):
        return alarm and alarm[CProps.SEVERITY] != 'OK'

    def _status_changed(self, new_alarm, old_alarm):
        return new_alarm and old_alarm \
            and not new_alarm[CProps.SEVERITY] == old_alarm[CProps.SEVERITY]

    def _is_valid(self, alarm):
        return alarm[CProps.RESOURCE_TYPE] is not None \
            and alarm[CProps.RESOURCE_NAME] is not None

    @staticmethod
    def _configuration_mapping():
        try:
            collectd_config_file = CONF.collectd[DSOpts.CONFIG_FILE]
            collectd_config = file_utils.load_yaml_file(collectd_config_file)
            collectd_config_elements = collectd_config[COLLECTD_DATASOURCE]

            mappings = {}
            for element_config in collectd_config_elements:
                mappings[element_config['collectd_host']] = {
                    CProps.RESOURCE_TYPE: element_config['type'],
                    CProps.RESOURCE_NAME: element_config['name']
                }

            LOG.debug('collectd mappings: %s', mappings)

            return mappings
        except Exception:
            LOG.exception('Failed in init.')
            return {}

    def enrich_event(self, event, event_type):
        event[DSProps.EVENT_TYPE] = event_type

        if CollectdDriver.conf_map:
            # PLUGIN_INSTANCE is optional
            resources = [event[CProps.HOST], event[CProps.PLUGIN],
                         event.get(CProps.PLUGIN_INSTANCE)]
            resource = '/'.join([resource for resource in resources if
                                 resource])
            v_resource = CollectdDriver.conf_map.find(resource)
            event[CProps.RESOURCE_NAME] = v_resource[CProps.RESOURCE_NAME]
            event[CProps.RESOURCE_TYPE] = v_resource[CProps.RESOURCE_TYPE]

        return CollectdDriver.make_pickleable([event], COLLECTD_DATASOURCE,
                                              DatasourceAction.UPDATE)[0]

    @staticmethod
    def get_event_types():
        return ['collectd.alarm.ok',
                'collectd.alarm.failure',
                'collectd.alarm.warning']

    @staticmethod
    def should_delete_outdated_entities():
        return True



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import functools
from oslo_log import log

from vitrage.common.constants import EdgeProperties as EProps
from vitrage.common.constants import EntityCategory
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources.nova.host import NOVA_HOST_DATASOURCE
from vitrage.datasources.nova.instance import NOVA_INSTANCE_DATASOURCE
from vitrage.datasources.nova.zone import NOVA_ZONE_DATASOURCE
from vitrage.datasources import OPENSTACK_CLUSTER

LOG = log.getLogger(__name__)


# Used for Sunburst to show only specific resources
TREE_TOPOLOGY_QUERY = {
    'and': [
        {'==': {VProps.VITRAGE_CATEGORY: EntityCategory.RESOURCE}},
        {'==': {VProps.VITRAGE_IS_DELETED: False}},
        {'==': {VProps.VITRAGE_IS_PLACEHOLDER: False}},
        {
            'or': [
                {'==': {VProps.VITRAGE_TYPE: OPENSTACK_CLUSTER}},
                {'==': {VProps.VITRAGE_TYPE: NOVA_INSTANCE_DATASOURCE}},
                {'==': {VProps.VITRAGE_TYPE: NOVA_HOST_DATASOURCE}},
                {'==': {VProps.VITRAGE_TYPE: NOVA_ZONE_DATASOURCE}}
            ]
        }
    ]
}

TOPOLOGY_AND_ALARMS_QUERY = {
    'and': [
        {'==': {VProps.VITRAGE_IS_DELETED: False}},
        {'==': {VProps.VITRAGE_IS_PLACEHOLDER: False}},
        {
            'or': [
                {'==': {VProps.VITRAGE_CATEGORY: EntityCategory.ALARM}},
                {'==': {VProps.VITRAGE_CATEGORY: EntityCategory.RESOURCE}}
            ]
        }
    ]
}


ALARMS_ALL_QUERY = {
    'and': [
        {'==': {VProps.VITRAGE_CATEGORY: EntityCategory.ALARM}},
        {'==': {VProps.VITRAGE_IS_DELETED: False}}
    ]
}

EDGE_QUERY = {'==': {EProps.VITRAGE_IS_DELETED: False}}

RESOURCES_ALL_QUERY = {
    'and': [
        {'==': {VProps.VITRAGE_CATEGORY: EntityCategory.RESOURCE}},
        {'==': {VProps.VITRAGE_IS_DELETED: False}},
        {'==': {VProps.VITRAGE_IS_PLACEHOLDER: False}}
    ]
}


class EntityGraphApisBase(object):

    def __init__(self, entity_graph, api_lock, db=None):
        self.entity_graph = entity_graph
        self.db = db
        self.api_lock = api_lock

    @classmethod
    def _get_query_with_project(cls, vitrage_category, project_id, is_admin):
        """Generate query with tenant data

        Creates query for entity graph which takes into consideration the
        vitrage_category, project_id and if the tenant is admin

        :type vitrage_category: string
        :type project_id: string
        :type is_admin: boolean
        :rtype: dictionary
        """

        query = {
            'and': [
                {'==': {VProps.VITRAGE_IS_DELETED: False}},
                {'==': {VProps.VITRAGE_IS_PLACEHOLDER: False}},
                {'==': {VProps.VITRAGE_CATEGORY: vitrage_category}}
            ]
        }

        cls._add_project_to_query(query, project_id, is_admin)

        return query

    @staticmethod
    def _add_project_to_query(query, project_id, is_admin):
        """Add project_id filter to the query

        Each query should contain the project_id condition

        :type query: string representing a json query
        :type project_id: string
        :type is_admin: boolean
        :rtype: string representing a json query
        """

        if is_admin:
            project_query = \
                {'or': [{'==': {VProps.PROJECT_ID: project_id}},
                        {'==': {VProps.PROJECT_ID: None}}]}
        else:
            project_query = \
                {'==': {VProps.PROJECT_ID: project_id}}

        if 'and' in query:
            query_with_project_id = query
            query_with_project_id['and'].append(project_query)
        else:
            query_with_project_id = {'and': [project_query, query]}

        return query_with_project_id


def lock_graph(f):
    @functools.wraps(f)
    def api_backend_func(*args, **kwargs):
        try:
            args[0].api_lock.acquire()
            result = f(*args, **kwargs)
            return result
        finally:
            args[0].api_lock.release()
    return api_backend_func



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from datetime import datetime
from datetime import timedelta
from dateutil import parser
from oslo_utils import timeutils


TIMESTAMP_FORMAT = '%Y-%m-%dT%H:%M:%SZ'


def utcnow(with_timezone=True):
    """Better version of utcnow() that returns utcnow with a correct TZ."""
    return timeutils.utcnow(with_timezone)


def format_utcnow(with_timezone=True, date_format=TIMESTAMP_FORMAT):
    return utcnow(with_timezone).strftime(date_format)


def change_time_str_format(timestamp_str, old_format, new_format):
    utc = datetime.strptime(timestamp_str, old_format)
    return utc.strftime(new_format)


def change_to_utc_time_and_format(timestamp_str, new_format):
    timestamp = parser.parse(timestamp_str)
    timestamp = timestamp - timedelta(seconds=(
        datetime.now() - datetime.utcnow()).total_seconds())
    return timestamp.strftime(new_format)


def format_unix_timestamp(timestamp, date_format=TIMESTAMP_FORMAT):
    return datetime.fromtimestamp(float(timestamp)) \
        .strftime(date_format)


def format_timestamp(timestamp_str, new_format=TIMESTAMP_FORMAT):
    return parser.parse(timestamp_str).strftime(new_format) if timestamp_str \
        else None


def datetime_delta(delta_seconds,
                   time_to_use=None,
                   with_timezone=True,
                   date_format=TIMESTAMP_FORMAT):
    if time_to_use:
        t = time_to_use
    else:
        t = utcnow(with_timezone)

    return (t + timedelta(seconds=delta_seconds)).strftime(date_format)



# Copyright 2015 - Alcatel-Lucent
# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from oslo_config import cfg
from oslo_log import log
import oslo_messaging as messaging
from oslo_messaging.rpc import dispatcher
from osprofiler import profiler


OPTS = [
    cfg.StrOpt('rpc_topic',
               default='rpcapiv1',
               help='The topic vitrage listens on'),
]

LOG = log.getLogger(__name__)


class ProfilerContextSerializer(messaging.Serializer):
    def __init__(self, serializer):
        self._serializer = serializer

    def serialize_entity(self, context, entity):
        if not self._serializer:
            return entity
        return self._serializer.serialize_entity(context, entity)

    def deserialize_entity(self, context, entity):
        if not self._serializer:
            return entity
        return self._serializer.deserialize_entity(context, entity)

    def serialize_context(self, context):
        ctx = self._serializer.serialize_context(context) \
            if self._serializer else context

        pfr = profiler.get()

        if pfr:
            ctx['trace_info'] = {
                "hmac_key": pfr.hmac_key,
                "base_id": pfr.get_base_id(),
                "parent_id": pfr.get_id()
            }

        return ctx

    def deserialize_context(self, context):
        trace_info = context.pop('trace_info', None)

        if trace_info:
            profiler.init(**trace_info)

        return self._serializer.deserialize_context(context)\
            if self._serializer else context


def set_defaults(control_exchange):
    messaging.set_transport_defaults(control_exchange)


def get_client(transport, target, version_cap=None, serializer=None):
    assert transport is not None

    if profiler:
        LOG.info('profiler enabled for RPC client')
        serializer = ProfilerContextSerializer(serializer=serializer)

    return messaging.RPCClient(transport,
                               target,
                               version_cap=version_cap,
                               serializer=serializer)


def get_server(target, endpoints, transport, serializer=None):
    assert transport is not None

    if profiler:
        LOG.debug('profiler enabled for RPC server')
        serializer = ProfilerContextSerializer(serializer=serializer)

    access_policy = dispatcher.DefaultRPCAccessPolicy
    return messaging.get_rpc_server(transport,
                                    target,
                                    endpoints,
                                    serializer=serializer,
                                    access_policy=access_policy)



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from collections import namedtuple
from vitrage.evaluator.template_validation.status_messages import status_msgs
RESULT_DESCRIPTION = 'Template syntax validation'
EXCEPTION = 'exception'

Result = namedtuple('Result', ['description', 'is_valid_config', 'status_code',
                               'comment'])


class ValidationError(Exception):
    def __init__(self, code, *args):
        self.code = code
        self.details = ''
        self.details = ','.join(str(arg) for arg in args)


def get_correct_result(description=RESULT_DESCRIPTION):
    return Result(description, True, 0, status_msgs[0])


def get_warning_result(description, code):
    return Result(description, True, code, status_msgs[code])


def get_fault_result(description, code, msg=None):
    if msg:
        return Result(description, False, code, msg)
    return Result(description, False, code, status_msgs[code])


def get_custom_fault_result(code, msg):
    return Result('Template validation', False, code,
                  status_msgs[code] + ' - ' + msg)


def get_status_code(voluptuous_error):
    prefix = str(voluptuous_error).split(' ')[0].strip()
    if prefix.isdigit():
        return int(prefix)
    return 4



# Copyright 2019 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from collections import namedtuple
from oslo_log import log
import re

from vitrage.evaluator.template_validation.base import ValidationError

LOG = log.getLogger(__name__)

FuncInfo = namedtuple('FuncInfo', ['name', 'func', 'error_code'])


def resolve_function(func_info, template, **kwargs):
    return _traverse_function(func_info, template, resolve=True, **kwargs)


def validate_function(func_info, template, **kwargs):
    return _traverse_function(func_info, template, resolve=False, **kwargs)


def _traverse_function(func_info, template, resolve, **kwargs):
    return _recursive_resolve_function(
        func_info, template, template, resolve, **kwargs)


def _recursive_resolve_function(func_info, template, template_block,
                                resolve, **kwargs):

    for key, value in template_block.items():
        if isinstance(value, str) and \
                _is_wanted_function(value, func_info.name):

            if not func_info.func:
                raise ValidationError(func_info.error_code, value)

            resolved_value = func_info.func(value, template, **kwargs)
            if resolve:
                template_block[key] = resolved_value
                LOG.debug('Replaced %s with %s', value, resolved_value)

        elif isinstance(value, dict):
            _recursive_resolve_function(
                func_info, template, value, resolve, **kwargs)

        elif isinstance(value, list):
            for item in value:
                _recursive_resolve_function(
                    func_info, template, item, resolve, **kwargs)


def is_function(str):
    """Check if the string represents a function

    A function has the format: func_name(params)
    Search for a regex with open and close parenthesis
    """
    return re.match(r'.*\(.*\)', str)


def _is_wanted_function(str, func_name):
    """Check if the string represents `func_name` function

    A function has the format: func_name(params)
    Search for a regex with open and close parenthesis
    """
    return re.match(func_name + r'\(.*\)', str)



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys

from oslo_config import cfg

from vitrage.cli import VITRAGE_TITLE
from vitrage.common import config
from vitrage import storage
from vitrage.storage.sqlalchemy import migration

CONF = cfg.CONF
CLI_OPTS = [
    cfg.StrOpt('revision',
               default='head',
               help='Migration version')
]
REVISION_OPTS = [
    cfg.StrOpt('message',
               help='Text that will be used for migration title'),
    cfg.BoolOpt('autogenerate',
                default=False,
                help='Generates diff based on current database state')
]


def stamp():
    print(VITRAGE_TITLE)
    CONF.register_cli_opts(CLI_OPTS)
    config.parse_config(sys.argv)

    migration.stamp(CONF.revision)


def revision():
    print(VITRAGE_TITLE)
    CONF.register_cli_opts(REVISION_OPTS)
    config.parse_config(sys.argv)
    migration.revision(CONF.message, CONF.autogenerate)


def dbsync():
    print(VITRAGE_TITLE)
    CONF.register_cli_opts(CLI_OPTS)
    config.parse_config(sys.argv)
    migration.upgrade(CONF.revision)


def purge_data():
    print(VITRAGE_TITLE)
    config.parse_config(sys.argv)
    db = storage.get_connection_from_config()
    db.active_actions.delete()
    db.events.delete()
    db.graph_snapshots.delete()
    db.changes.delete()
    db.edges.delete()
    db.alarms.delete()



# Copyright 2017 - ZTE, Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_config import cfg
import testtools

from vitrage.common.constants import DatasourceOpts as DSOpts
from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import UpdateMethod
from vitrage.datasources.ceilometer import CEILOMETER_DATASOURCE
from vitrage.datasources.ceilometer.properties import CeilometerEventType
from vitrage.datasources.ceilometer.properties \
    import CeilometerProperties as CeilProps
from vitrage.tests import base
from vitrage.tests.mocks import mock_driver
from vitrage.tests.unit.datasources.ceilometer.mock_driver \
    import MockCeilometerDriver


@testtools.skip("skip for now")
class CeilometerDriverTest(base.BaseTest):

    OPTS = [
        cfg.StrOpt(DSOpts.UPDATE_METHOD,
                   default=UpdateMethod.PUSH),
    ]

    # noinspection PyPep8Naming
    @classmethod
    def setUpClass(cls):
        super(CeilometerDriverTest, cls).setUpClass()
        cls.conf = cfg.ConfigOpts()
        cls.conf.register_opts(cls.OPTS, group=CEILOMETER_DATASOURCE)

    def test_event_alarm_notifications(self):

        aodh_driver = MockCeilometerDriver()

        # 1. alarm creation with 'ok' state
        # prepare data
        detail_data = {"type": "creation",
                       CeilProps.DETAIL: self._create_alarm_data_type_event(),
                       }
        generators = \
            mock_driver.simple_aodh_alarm_notification_generators(
                alarm_num=1,
                update_events=1,
                update_vals=detail_data)
        alarm = mock_driver.generate_sequential_events_list(generators)[0]
        alarm_info = alarm.copy()

        # action
        entity = aodh_driver.enrich_event(alarm, CeilometerEventType.CREATION)

        # Test assertions
        # alarm with status OK should not be handled
        self.assertIsNone(entity)

        # 2.alarm state transition from 'ok' to 'alarm'
        detail_data = {"type": "state transition",
                       CeilProps.DETAIL: {CeilProps.STATE: "alarm"}}
        alarm.update(detail_data)
        entity = aodh_driver.enrich_event(alarm,
                                          CeilometerEventType.STATE_TRANSITION)

        # Test assertions
        # alarm state change: ok->alarm, need to be added
        self.assertIsNotNone(entity)
        self._validate_aodh_entity_comm_props(entity, alarm_info)
        self.assertEqual(entity[CeilProps.STATE],
                         alarm[CeilProps.DETAIL][CeilProps.STATE])
        self.assertEqual(entity[CeilProps.SEVERITY],
                         alarm[CeilProps.SEVERITY])
        self.assertEqual(entity[DSProps.EVENT_TYPE],
                         CeilometerEventType.STATE_TRANSITION)

        # 3. delete alarm which is 'alarm' state
        # prepare data
        detail_data = {"type": "deletion"}
        alarm.update(detail_data)

        # action
        entity = aodh_driver.enrich_event(
            alarm, CeilometerEventType.DELETION)

        # Test assertions
        self.assertIsNotNone(entity)
        self._validate_aodh_entity_comm_props(entity, alarm_info)
        self.assertEqual(entity[DSProps.EVENT_TYPE],
                         CeilometerEventType.DELETION)

        # 4. alarm creation with 'alarm' state
        # prepare data
        detail_data = {"type": "creation",
                       CeilProps.DETAIL:
                           self._create_alarm_data_type_event(state="alarm")}
        generators = \
            mock_driver.simple_aodh_alarm_notification_generators(
                alarm_num=1,
                update_events=1,
                update_vals=detail_data)
        alarm = mock_driver.generate_sequential_events_list(generators)[0]
        alarm_info = alarm.copy()

        # action
        entity = aodh_driver.enrich_event(
            alarm, CeilometerEventType.CREATION)

        # Test assertions
        # alarm with status 'alarm' need to be added
        self.assertIsNotNone(entity)
        self._validate_aodh_entity_comm_props(entity, alarm_info)
        self.assertEqual(entity[CeilProps.STATE],
                         alarm[CeilProps.DETAIL][CeilProps.STATE])
        self.assertEqual(entity[CeilProps.SEVERITY],
                         alarm[CeilProps.SEVERITY])
        self.assertIsNone(entity[CeilProps.RESOURCE_ID])
        self.assertEqual("*", entity[CeilProps.EVENT_TYPE])
        self.assertEqual(entity[DSProps.EVENT_TYPE],
                         CeilometerEventType.CREATION)

        # 5. alarm rule change
        # prepare data
        detail_data = {"type": "rule change",
                       CeilProps.DETAIL: {
                           "severity": "critical",
                           CeilProps.RULE:
                               {"query": [{"field": "traits.resource_id",
                                           "type": "",
                                           "value": "1",
                                           "op": "eq"}],
                                "event_type": "instance.update"}}}
        alarm.update(detail_data)

        # action
        entity = aodh_driver.enrich_event(
            alarm, CeilometerEventType.RULE_CHANGE)

        # Test assertions
        # alarm rule change: need to be update
        self.assertIsNotNone(entity)
        self._validate_aodh_entity_comm_props(entity, alarm_info)
        self.assertEqual(entity[CeilProps.SEVERITY],
                         alarm[CeilProps.DETAIL][CeilProps.SEVERITY])
        self.assertEqual(
            entity[CeilProps.EVENT_TYPE],
            alarm[CeilProps.DETAIL][CeilProps.RULE][CeilProps.EVENT_TYPE])
        self.assertEqual("1", entity[CeilProps.RESOURCE_ID])
        self.assertEqual(entity[DSProps.EVENT_TYPE],
                         CeilometerEventType.RULE_CHANGE)

        # 6. alarm state change from 'alarm' to 'ok'
        # prepare data
        detail_data = {"type": "state transition",
                       CeilProps.DETAIL: {CeilProps.STATE: "ok"}}
        alarm.update(detail_data)

        # action
        entity = aodh_driver.enrich_event(
            alarm, CeilometerEventType.STATE_TRANSITION)

        # Test assertions
        # alarm state change: alarm->OK, need to be deleted
        self.assertIsNotNone(entity)
        self._validate_aodh_entity_comm_props(entity, alarm_info)
        self.assertEqual(entity[DSProps.EVENT_TYPE],
                         CeilometerEventType.STATE_TRANSITION)

        # 7. delete alarm which is 'ok' state
        # prepare data
        detail_data = {"type": "deletion"}
        alarm.update(detail_data)

        # action
        entity = aodh_driver.enrich_event(
            alarm, CeilometerEventType.DELETION)

        # Test assertions
        self.assertIsNone(entity)

    def test_gnocchi_threshold_alarm_notifications(self):
        aodh_driver = MockCeilometerDriver()

        # 1. alarm creation with 'ok' state
        # prepare data
        detail_data = {"type": "gnocchi_resources_threshold",
                       CeilProps.DETAIL: self._create_alarm_data_gnocchi()}
        generators = \
            mock_driver.simple_aodh_alarm_notification_generators(
                alarm_num=1,
                update_events=1,
                update_vals=detail_data)
        alarm = mock_driver.generate_sequential_events_list(generators)[0]
        alarm_info = alarm.copy()

        # action
        entity = aodh_driver.enrich_event(
            alarm, CeilometerEventType.CREATION)

        # Test assertions
        # alarm with status OK should not be handled
        self.assertIsNone(entity)

        # 2.alarm state transition from 'ok' to 'alarm'
        detail_data = {"type": "state transition",
                       CeilProps.DETAIL: {CeilProps.STATE: "alarm"}}
        alarm.update(detail_data)
        entity = aodh_driver.enrich_event(
            alarm, CeilometerEventType.STATE_TRANSITION)

        # Test assertions
        # alarm state change: ok->alarm, need to be added
        self.assertIsNotNone(entity)
        self._validate_aodh_entity_comm_props(entity, alarm_info)
        self.assertEqual(entity[CeilProps.STATE],
                         alarm[CeilProps.DETAIL][CeilProps.STATE])
        self.assertEqual(entity[CeilProps.SEVERITY],
                         alarm[CeilProps.SEVERITY])

        # 3. delete alarm which is 'alarm' state
        # prepare data
        detail_data = {"type": "deletion"}
        alarm.update(detail_data)

        # action
        entity = aodh_driver.enrich_event(
            alarm, CeilometerEventType.DELETION)

        # Test assertions
        self.assertIsNotNone(entity)
        self._validate_aodh_entity_comm_props(entity, alarm_info)
        self.assertEqual(entity[DSProps.EVENT_TYPE],
                         CeilometerEventType.DELETION)

        # 4. alarm creation with 'alarm' state
        # prepare data
        detail_data = {"type": "gnocchi_resources_threshold",
                       CeilProps.DETAIL:
                           self._create_alarm_data_gnocchi(state="alarm")}
        generators = \
            mock_driver.simple_aodh_alarm_notification_generators(
                alarm_num=1,
                update_events=1,
                update_vals=detail_data)
        alarm = mock_driver.generate_sequential_events_list(generators)[0]
        alarm_info = alarm.copy()

        # action
        entity = aodh_driver.enrich_event(
            alarm, CeilometerEventType.CREATION)

        # Test assertions
        # alarm with status 'alarm' need to be added
        self.assertIsNotNone(entity)
        self._validate_aodh_entity_comm_props(entity, alarm_info)
        self.assertEqual(entity[CeilProps.STATE],
                         alarm[CeilProps.DETAIL][CeilProps.STATE])
        self.assertEqual(entity[CeilProps.SEVERITY],
                         alarm[CeilProps.SEVERITY])
        self.assertEqual(entity[DSProps.EVENT_TYPE],
                         CeilometerEventType.CREATION)

        # 5. alarm rule change
        # prepare data
        detail_data = {"type": "rule change",
                       CeilProps.DETAIL: {
                           "severity": "critical",
                           CeilProps.RULE:
                               {"granularity": "300",
                                "threshold": "0.0123",
                                "comparison_operator": "eq"}}}
        alarm.update(detail_data)

        # action
        entity = aodh_driver.enrich_event(
            alarm, CeilometerEventType.RULE_CHANGE)

        # Test assertions
        # alarm rule change: need to be update
        self.assertIsNotNone(entity)
        self._validate_aodh_entity_comm_props(entity, alarm_info)
        self.assertEqual(entity[CeilProps.SEVERITY],
                         alarm[CeilProps.DETAIL][CeilProps.SEVERITY])
        self.assertEqual(entity[DSProps.EVENT_TYPE],
                         CeilometerEventType.RULE_CHANGE)

        # 6. alarm state change from 'alarm' to 'ok'
        # prepare data
        detail_data = {"type": "state transition",
                       CeilProps.DETAIL: {CeilProps.STATE: "ok"}}
        alarm.update(detail_data)

        # action
        entity = aodh_driver.enrich_event(
            alarm,
            CeilometerEventType.STATE_TRANSITION)

        # Test assertions
        # alarm state change: alarm->OK, need to be deleted
        self.assertIsNotNone(entity)
        self._validate_aodh_entity_comm_props(entity, alarm_info)
        self.assertEqual(entity[DSProps.EVENT_TYPE],
                         CeilometerEventType.STATE_TRANSITION)

        # 7. delete alarm which is 'ok' state
        # prepare data
        detail_data = {"type": "deletion"}
        alarm.update(detail_data)

        # action
        entity = aodh_driver.enrich_event(
            alarm, CeilometerEventType.DELETION)

        # Test assertions
        self.assertIsNone(entity)

    def _create_alarm_data_gnocchi(self,
                                   state="ok",
                                   type="gnocchi_resources_threshold",
                                   rule=None):

        if rule is None:
            rule = {"granularity": "300",
                    "threshold": "0.001",
                    "comparison_operator": "gt",
                    "resource_type": "instance"
                    }
        return {CeilProps.DESCRIPTION: "test",
                CeilProps.TIMESTAMP: "2016-11-09T01:39:13.839584",
                CeilProps.ENABLED: True,
                CeilProps.STATE_TIMESTAMP: "2016-11-09T01:39:13.839584",
                CeilProps.ALARM_ID: "7e5c3754-e2eb-4782-ae00-7da5ded8568b",
                CeilProps.REPEAT_ACTIONS: False,
                CeilProps.PROJECT_ID: "c365d18fcc03493187016ae743f0cc4d",
                CeilProps.NAME: "test",
                CeilProps.SEVERITY: "low",
                CeilProps.RESOURCE_ID: "88cd2d1d-8af4-4d00-9b5e-f82f8c8b0f8d",
                CeilProps.TYPE: type,
                CeilProps.STATE: state,
                CeilProps.RULE: rule,
                CeilProps.STATE_REASON: 'for test'}

    def _create_alarm_data_type_event(self,
                                      state="ok",
                                      type="event",
                                      rule=None):

        if rule is None:
            rule = {"query": [], "event_type": "*"}
        return {CeilProps.DESCRIPTION: "test",
                CeilProps.TIMESTAMP: "2016-11-09T01:39:13.839584",
                CeilProps.ENABLED: True,
                CeilProps.STATE_TIMESTAMP: "2016-11-09T01:39:13.839584",
                CeilProps.ALARM_ID: "7e5c3754-e2eb-4782-ae00-7da5ded8568b",
                CeilProps.REPEAT_ACTIONS: False,
                CeilProps.PROJECT_ID: "c365d18fcc03493187016ae743f0cc4d",
                CeilProps.NAME: "test",
                CeilProps.SEVERITY: "low",
                CeilProps.TYPE: type,
                CeilProps.STATE: state,
                CeilProps.RULE: rule,
                CeilProps.STATE_REASON: 'for test'}

    def _validate_aodh_entity_comm_props(self, entity, alarm):

        self.assertEqual(entity[CeilProps.ALARM_ID],
                         alarm[CeilProps.ALARM_ID])
        self.assertEqual(entity[CeilProps.PROJECT_ID],
                         alarm[CeilProps.PROJECT_ID])
        self.assertEqual(entity[CeilProps.TIMESTAMP],
                         alarm[CeilProps.TIMESTAMP])
        self.assertEqual(entity[CeilProps.DESCRIPTION],
                         alarm[CeilProps.DETAIL][CeilProps.DESCRIPTION])
        self.assertEqual(entity[CeilProps.ENABLED],
                         alarm[CeilProps.DETAIL][CeilProps.ENABLED])
        self.assertEqual(entity[CeilProps.NAME],
                         alarm[CeilProps.DETAIL][CeilProps.NAME])
        self.assertEqual(entity[CeilProps.REPEAT_ACTIONS],
                         alarm[CeilProps.DETAIL][CeilProps.REPEAT_ACTIONS])
        self.assertEqual(entity[CeilProps.TYPE],
                         alarm[CeilProps.DETAIL][CeilProps.TYPE])



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc


class NotifierBase(object, metaclass=abc.ABCMeta):

    @abc.abstractmethod
    def process_event(self, data, event_type):
        pass

    @staticmethod
    @abc.abstractmethod
    def get_notifier_name():
        pass

    @staticmethod
    def use_private_topic():
        return False

    @staticmethod
    def info(self, ctxt, publisher_id, event_type, payload, metadata):
        """An endpoint for notifiers that use a private topic"""
        pass



# Copyright 2020 - Inspur - Qitao
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import datetime

from oslo_config import cfg
from oslo_log import log as logging

from testtools import matchers

from vitrage.common.constants import DatasourceAction
from vitrage.common.constants import DatasourceOpts as DSOpts
from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import EntityCategory
from vitrage.common.constants import UpdateMethod
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources.cetus.pod import CETUS_POD_DATASOURCE
from vitrage.datasources.cetus.pod.transformer import PodTransformer
from vitrage.datasources.nova.instance import NOVA_INSTANCE_DATASOURCE
from vitrage.datasources import transformer_base as tbase
from vitrage.datasources.transformer_base import TransformerBase
from vitrage.tests import base

from vitrage.utils.datetime import format_utcnow

LOG = logging.getLogger(__name__)

events = [
    {
        'name': 'app01-3232323232',
        'id': '0903912039123',
        'status': 'Pending',
        'node': 'd7f8b66d-5221-4b4c-ade8-416a6a7bc661'
    }, {
        'name': 'app02-3232323232',
        'id': '0903912039122',
        'status': 'Pending',
        'node': '85ac015b-ec84-4d3c-a56e-d97fafff6a2a'
    }]


class TestCetusClusterTransformer(base.BaseTest):
    OPTS = [
        cfg.StrOpt(DSOpts.UPDATE_METHOD,
                   default=UpdateMethod.PULL),
    ]

    # noinspection PyAttributeOutsideInit,PyPep8Naming
    @classmethod
    def setUpClass(cls):
        super(TestCetusClusterTransformer, cls).setUpClass()
        cls.transformers = {}
        cls.conf = cfg.ConfigOpts()
        cls.conf.register_opts(cls.OPTS, group=CETUS_POD_DATASOURCE)
        cls.transformers[CETUS_POD_DATASOURCE] = PodTransformer(
            cls.transformers)

    def test_create_placeholder_vertex(self):
        LOG.debug('Cetus pod transformer test: Test create placeholder '
                  'vertex')

        # Test setup
        pod_id = "pod123"
        timestamp = datetime.datetime.utcnow()
        pod_transformer = self.transformers[CETUS_POD_DATASOURCE]

        # Test action
        properties = {
            VProps.ID: pod_id,
            VProps.VITRAGE_TYPE: CETUS_POD_DATASOURCE,
            VProps.VITRAGE_CATEGORY: EntityCategory.RESOURCE,
            VProps.VITRAGE_SAMPLE_TIMESTAMP: timestamp
        }
        placeholder = \
            pod_transformer.create_neighbor_placeholder_vertex(**properties)

        # Test assertions
        observed_uuid = placeholder.vertex_id
        expected_key = tbase.build_key(pod_transformer._key_values(
            CETUS_POD_DATASOURCE,
            pod_id))
        expected_uuid = \
            TransformerBase.uuid_from_deprecated_vitrage_id(expected_key)
        self.assertEqual(expected_uuid, observed_uuid)

        observed_time = placeholder.get(VProps.VITRAGE_SAMPLE_TIMESTAMP)
        self.assertEqual(timestamp, observed_time)

        observed_subtype = placeholder.get(VProps.VITRAGE_TYPE)
        self.assertEqual(CETUS_POD_DATASOURCE, observed_subtype)

        observed_entity_id = placeholder.get(VProps.ID)
        self.assertEqual(pod_id, observed_entity_id)

        observed_vitrage_category = placeholder.get(VProps.VITRAGE_CATEGORY)
        self.assertEqual(EntityCategory.RESOURCE, observed_vitrage_category)

        vitrage_is_placeholder = placeholder.get(
            VProps.VITRAGE_IS_PLACEHOLDER)
        self.assertTrue(vitrage_is_placeholder)

    def test_key_values(self):

        LOG.debug('Test key values')

        # Test setup
        pod_id = "pod123456"
        pod_transformer = self.transformers[CETUS_POD_DATASOURCE]
        # Test action
        observed_key_fields = pod_transformer._key_values(
            CETUS_POD_DATASOURCE,
            pod_id)

        # Test assertions
        self.assertEqual(EntityCategory.RESOURCE, observed_key_fields[0])
        self.assertEqual(CETUS_POD_DATASOURCE, observed_key_fields[1])
        self.assertEqual(pod_id, observed_key_fields[2])

    def test_snapshot_event_transform(self):
        sample_timestamp = format_utcnow()
        for event in events:
            event[DSProps.DATASOURCE_ACTION] = DatasourceAction.SNAPSHOT
            event[DSProps.SAMPLE_DATE] = sample_timestamp
            wrapper = self.transformers[CETUS_POD_DATASOURCE].transform(event)
            vertex = wrapper.vertex
            self._validate_vertex_props(vertex, event)
            neighbors = wrapper.neighbors
            self.assertThat(neighbors, matchers.HasLength(1))
            self._validate_neighbors(neighbors, vertex.vertex_id)

    def _validate_neighbors(self, neighbors, instance_vertex_id):
        node_neighbors_counter = 0
        for neighbor in neighbors:
            node_neighbors_counter += 1
            self._validate_node_neighbor(neighbor, instance_vertex_id)

        self.assertEqual(1, node_neighbors_counter)

    def _validate_node_neighbor(self, node_neighbor, instance_vertex_id):

        self.assertEqual(node_neighbor.vertex[VProps.VITRAGE_ID],
                         node_neighbor.vertex.vertex_id)
        self.assertFalse(node_neighbor.vertex[VProps.VITRAGE_IS_DELETED])

        self.assertTrue(node_neighbor.vertex[VProps.VITRAGE_IS_PLACEHOLDER])
        self.assertEqual(NOVA_INSTANCE_DATASOURCE,
                         node_neighbor.vertex[VProps.VITRAGE_TYPE])

        # Validate neighbor edge
        edge = node_neighbor.edge
        self.assertEqual(edge.source_id, node_neighbor.vertex.vertex_id)
        self.assertEqual(edge.target_id, instance_vertex_id)

    def _validate_vertex_props(self, vertex, event):

        extract_value = tbase.extract_field_value

        expected_id = extract_value(event, 'id')
        observed_id = vertex[VProps.ID]
        self.assertEqual(expected_id, observed_id)

        self.assertEqual(EntityCategory.RESOURCE,
                         vertex[VProps.VITRAGE_CATEGORY])

        self.assertEqual(CETUS_POD_DATASOURCE,
                         vertex[VProps.VITRAGE_TYPE])

        expected_timestamp = event[DSProps.SAMPLE_DATE]
        observed_timestamp = vertex[VProps.VITRAGE_SAMPLE_TIMESTAMP]
        self.assertEqual(expected_timestamp, observed_timestamp)

        expected_name = extract_value(event, 'name')
        observed_name = vertex[VProps.NAME]
        self.assertEqual(expected_name, observed_name)

        vitrage_is_placeholder = vertex[VProps.VITRAGE_IS_PLACEHOLDER]
        self.assertFalse(vitrage_is_placeholder)

        vitrage_is_deleted = vertex[VProps.VITRAGE_IS_DELETED]
        self.assertFalse(vitrage_is_deleted)



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from vitrage.common.constants import EntityCategory
from vitrage.common.constants import TemplateTypes
from vitrage.evaluator.actions.base import action_types
from vitrage.evaluator.template_schema_factory import TemplateSchemaFactory


status_msgs = {

    # General 1-19
    0: 'Template validation is OK',
    1: 'template_id field contains incorrect string value.',
    2: 'Duplicate template_id definition.',
    3: 'template_id does not appear in the definition block.',
    4: 'Syntax error: ',
    5: 'Invalid yaml format: ',

    # definitions section 20-39
    20: 'definitions section must contain entities field.',
    21: 'definitions section is a mandatory section.',

    # Entities status messages 40-59
    41: 'Entity definition must contain template_id field.',
    42: 'Entity definition must contain category field.',
    43: 'At least one entity must be defined.',
    45: 'Invalid entity category. Category must be from types: '
        '{categories}'.format(categories=EntityCategory.categories()),
    46: 'Entity field is required.',
    47: 'Invalid regular expression defined in field:',

    # metadata section status messages 60-79
    60: 'metadata section must contain id field.',
    62: 'metadata is a mandatory section.',
    63: 'Unsupported version. Version must be one of: {versions}'
        .format(versions=TemplateSchemaFactory.supported_versions()),
    64: 'metadata section must contain a type field starting from version 2. '
        'Type must be one of: {types}'.format(types=TemplateTypes.types()),
    65: 'Invalid template type. Type must be one of: {types}'
        .format(types=TemplateTypes.types()),
    66: 'Missing template type. Type must be one of: {types}'
        .format(types=TemplateTypes.types()),

    # scenarios section status messages 80-99
    80: 'scenarios is a mandatory section.',
    81: 'At least one scenario must be defined.',
    82: 'scenario field is required.',
    83: 'Scenario definition must contain condition field.',
    84: 'Scenario definition must contain actions field.',
    85: 'Failed to convert condition.',
    86: 'Not operator can be used only on relationships.',

    # relationships status messages 100-119
    101: 'Relationship field is required.',
    102: 'Relationship definition must contain source field.',
    103: 'Relationship definition must contain target field.',
    104: 'Relationship definition must contain template_id field.',

    # actions status messages 120-139
    120: 'Invalid action type. Action type must be from types: '
         '{actions}'.format(actions=action_types),
    121: 'At least one action must be defined.',
    122: 'Action field is required.',
    123: 'Action definition must contain action_type field.',
    124: 'Action definition must contain action_target field.',
    125: 'raise_alarm action must contain alarm_name field in properties '
         'block.',
    126: 'raise_alarm action must contain severity field in properties block.',
    127: 'raise_alarm action must contain target field in target_action block',
    128: 'set_state action must contain state field in properties block.',
    129: 'set_state action must contain target field in target_action block.',
    130: 'add_causal_relationship action must contain target and source field '
         'in target_action block.',
    131: 'mark_down action must contain \'target\' field in'
         ' \'target_action\' block.',
    132: 'add_causal_relationship action requires action_target to be ALARM',
    133: 'execute_mistral action must contain workflow field in properties '
         'block',
    134: 'condition can not contain only \'not\' clauses',
    135: 'condition must contain a common entity for all \'or\' clauses',
    136: 'Input parameters for the Mistral workflow in execute_mistral action '
         'must be placed under an \'input\' block ',
    137: 'Functions are supported only from version 2',
    138: 'Only open or close parenthesis exists. Did you try to use '
         'a function?',

    # def_templates status messages 140-159
    140: 'At least one template must be included',
    141: 'Name field is unspecified for include',
    142: 'Trying to include a template that does not exist',
    143: 'A template definition file cannot contain \'includes\' or '
         '\'scenarios\' blocks',

    # parameters status messages 160-179
    160: 'Parameters are supported only from version 2',
    161: 'get_param called for a parameter that is not defined in the '
         '\'parameters\' block',
    162: 'malformed get_param() clause',
    163: 'Failed to resolve parameter',

    # template version 3
    10100: 'Action must contain a \'target\' property',
    10101: 'Action \'target\' must match an entity id',
    10102: 'Action must contain a \'source\' property',
    10103: 'Action \'source\' must match an entity id',
    10104: 'raise_alarm action must contain an \'alarm_name\' property',
    10105: 'execute_mistral action must contain \'workflow\' property',
    10107: 'The property \'causing_alarm\' in raise_alarm action must match an'
           ' entity id',
    10200: 'Condition contains an unknown entity id',
}



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import datetime

from oslo_config import cfg

from vitrage.common.constants import DatasourceOpts as DSOpts
from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import EntityCategory
from vitrage.common.constants import UpdateMethod
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources.nova.host import NOVA_HOST_DATASOURCE
from vitrage.datasources.nova.host.transformer import HostTransformer
from vitrage.datasources.static import STATIC_DATASOURCE
from vitrage.datasources.static import StaticFields
from vitrage.datasources.static.transformer import StaticTransformer
from vitrage.datasources.transformer_base import TransformerBase
from vitrage.tests import base
from vitrage.tests.mocks import mock_driver


class TestStaticTransformer(base.BaseTest):

    OPTS = [
        cfg.StrOpt(DSOpts.UPDATE_METHOD,
                   default=UpdateMethod.PULL),
    ]

    # noinspection PyAttributeOutsideInit,PyPep8Naming
    @classmethod
    def setUpClass(cls):
        super(TestStaticTransformer, cls).setUpClass()
        cls.transformers = {}
        cls.conf = cfg.ConfigOpts()
        cls.conf.register_opts(cls.OPTS, group=STATIC_DATASOURCE)
        cls.transformer = StaticTransformer(cls.transformers)
        cls.transformers[STATIC_DATASOURCE] = cls.transformer
        cls.transformers[NOVA_HOST_DATASOURCE] = \
            HostTransformer(cls.transformers)

    # noinspection PyAttributeOutsideInit
    def setUp(self):
        super(TestStaticTransformer, self).setUp()
        self.entity_type = STATIC_DATASOURCE
        self.entity_id = '12345'
        self.timestamp = datetime.datetime.utcnow()

    def test_create_placeholder_vertex(self):
        properties = {
            VProps.VITRAGE_TYPE: self.entity_type,
            VProps.ID: self.entity_id,
            VProps.VITRAGE_CATEGORY: EntityCategory.RESOURCE,
            VProps.VITRAGE_SAMPLE_TIMESTAMP: self.timestamp
        }
        placeholder = self.transformer.create_neighbor_placeholder_vertex(
            **properties)

        observed_entity_id = placeholder.vertex_id
        expected_entity_id = \
            TransformerBase.uuid_from_deprecated_vitrage_id(
                'RESOURCE:static:12345')
        self.assertEqual(expected_entity_id, observed_entity_id)

        observed_time = placeholder.get(VProps.VITRAGE_SAMPLE_TIMESTAMP)
        self.assertEqual(self.timestamp, observed_time)

        observed_subtype = placeholder.get(VProps.VITRAGE_TYPE)
        self.assertEqual(self.entity_type, observed_subtype)

        observed_entity_id = placeholder.get(VProps.ID)
        self.assertEqual(self.entity_id, observed_entity_id)

        observed_vitrage_category = placeholder.get(VProps.VITRAGE_CATEGORY)
        self.assertEqual(EntityCategory.RESOURCE, observed_vitrage_category)

        vitrage_is_placeholder = placeholder.get(VProps.VITRAGE_IS_PLACEHOLDER)
        self.assertTrue(vitrage_is_placeholder)

    def test_snapshot_transform(self):
        vals_list = mock_driver.simple_static_generators(snapshot_events=1)
        events = mock_driver.generate_random_events_list(vals_list)
        self._event_transform_test(events)

    def test_update_transform(self):
        vals_list = mock_driver.simple_static_generators(update_events=1)
        events = mock_driver.generate_random_events_list(vals_list)
        self._event_transform_test(events)

    def _event_transform_test(self, events):
        for event in events:
            wrapper = self.transformer.transform(event)

            vertex = wrapper.vertex
            self._validate_vertex(vertex, event)

            neighbors = wrapper.neighbors
            self._validate_neighbors(neighbors, vertex.vertex_id, event)

    def _validate_vertex(self, vertex, event):
        self._validate_common_props(vertex, event)
        self.assertEqual(vertex[VProps.VITRAGE_SAMPLE_TIMESTAMP],
                         event[DSProps.SAMPLE_DATE])

        for k, v in event.get(StaticFields.METADATA, {}):
            self.assertEqual(v, vertex[k])

    def _validate_common_props(self, vertex, event):
        self.assertEqual(vertex[VProps.VITRAGE_CATEGORY],
                         EntityCategory.RESOURCE)
        self.assertEqual(vertex[VProps.VITRAGE_TYPE],
                         event[StaticFields.TYPE])
        self.assertEqual(vertex[VProps.ID], event[StaticFields.ID])
        self.assertFalse(vertex[VProps.VITRAGE_IS_DELETED])

    def _validate_neighbors(self, neighbors, vertex_id, event):
        for i in range(len(neighbors)):
            self._validate_neighbor(
                neighbors[i],
                event[StaticFields.RELATIONSHIPS][i],
                vertex_id)

    def _validate_neighbor(self, neighbor, rel, vertex_id):
        vertex = neighbor.vertex
        self._validate_neighbor_vertex_props(
            vertex,
            rel[StaticFields.TARGET])

        edge = neighbor.edge
        self.assertEqual(edge.source_id, vertex_id)
        self.assertEqual(edge.target_id, neighbor.vertex.vertex_id)
        self.assertEqual(edge.label,
                         rel[StaticFields.RELATIONSHIP_TYPE])

    def _validate_neighbor_vertex_props(self, vertex, event):
        self._validate_common_props(vertex, event)
        self.assertTrue(vertex[VProps.VITRAGE_IS_PLACEHOLDER])



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log

from vitrage.evaluator.template_data import EdgeDescription
from vitrage.evaluator.template_data import TemplateData
from vitrage.evaluator.template_fields import TemplateFields as TFields
from vitrage.evaluator.template_loading.props_converter import PropsConverter
from vitrage.evaluator.template_loading.scenario_loader import ScenarioLoader
from vitrage.graph import Edge
from vitrage.graph import Vertex
from vitrage.utils import evaluator as evaluator_utils


LOG = log.getLogger(__name__)


class TemplateLoader(object):

    def __init__(self):
        self.entities = {}
        self.relationships = {}

    def load(self, template_schema, template_def, def_templates=None):

        name = template_def[TFields.METADATA][TFields.NAME]

        # template_type might be None, it is defined only in version 2
        template_type = template_def[TFields.METADATA].get(TFields.TYPE)

        if def_templates is None:
            def_templates = {}
        defs = {}

        if TFields.DEFINITIONS in template_def:
            defs = template_def[TFields.DEFINITIONS]
            if TFields.ENTITIES in defs:
                self.entities = self._build_entities(defs[TFields.ENTITIES])

        # Add definitions from template then from definition templates.
        if TFields.INCLUDES in template_def:
            includes = template_def[TFields.INCLUDES]
            self._build_entities_from_def_templates(
                includes, def_templates, self.entities)

        if TFields.RELATIONSHIPS in defs:
            self.relationships = self._build_relationships(
                defs[TFields.RELATIONSHIPS])

        if TFields.INCLUDES in template_def:
            includes = template_def[TFields.INCLUDES]
            self._build_relationships_with_def_templates(includes,
                                                         def_templates,
                                                         self.relationships)

        scenarios = ScenarioLoader(template_schema, name, self.entities,
                                   self.relationships).\
            build_scenarios(template_def[TFields.SCENARIOS])

        return TemplateData(name, template_type, template_schema.version(),
                            self.entities, self.relationships, scenarios)

    def _build_entities(self, entities_defs):
        entities = {}
        for entity_def in entities_defs:

            entity_dict = entity_def[TFields.ENTITY]
            template_id = entity_dict[TFields.TEMPLATE_ID]
            properties = PropsConverter.convert_props_with_dictionary(
                self._extract_properties(entity_dict))
            entities[template_id] = Vertex(template_id, properties)

        return entities

    def _build_entities_from_def_templates(
            self, includes, def_templates, entities):

        for def_template_dict in includes:

            name = def_template_dict[TFields.NAME]
            def_template = evaluator_utils.find_def_template(
                name, def_templates)
            defs = def_template[TFields.DEFINITIONS]
            entities_defs = defs[TFields.ENTITIES]

            for entity_def in entities_defs:

                entity_dict = entity_def[TFields.ENTITY]
                template_id = entity_dict[TFields.TEMPLATE_ID]
                if template_id not in entities:

                    properties = \
                        PropsConverter.convert_props_with_dictionary(
                            self._extract_properties(entity_dict))
                    entities[template_id] = Vertex(template_id, properties)

    def _build_relationships(self, relationships_defs):

        relationships = {}
        for relationship_def in relationships_defs:

            relationship_dict = relationship_def[TFields.RELATIONSHIP]
            relationship = self._extract_relationship_info(relationship_dict)
            template_id = relationship_dict[TFields.TEMPLATE_ID]
            relationships[template_id] = relationship

        return relationships

    def _build_relationships_with_def_templates(
            self, includes, def_templates, relationships):

        for def_template_dict in includes:

            name = def_template_dict[TFields.NAME]
            def_template = evaluator_utils.find_def_template(
                name, def_templates)

            if TFields.RELATIONSHIPS in def_template[TFields.DEFINITIONS]:
                defs = def_template[TFields.DEFINITIONS]
                relationship_defs = defs[TFields.RELATIONSHIPS]

                for relationship_def in relationship_defs:
                    relationship_dict = relationship_def[TFields.RELATIONSHIP]
                    template_id = relationship_dict[TFields.TEMPLATE_ID]

                    if template_id not in relationships:
                        relationship = self._extract_relationship_info(
                            relationship_dict)
                        relationships[template_id] = relationship

    def _extract_relationship_info(self, relationship_dict):
        source_id = relationship_dict[TFields.SOURCE]
        target_id = relationship_dict[TFields.TARGET]

        edge = Edge(source_id,
                    target_id,
                    relationship_dict[TFields.RELATIONSHIP_TYPE],
                    self._extract_properties(relationship_dict))

        source = self.entities[source_id]
        target = self.entities[target_id]
        return EdgeDescription(edge, source, target)

    @staticmethod
    def _extract_properties(var_dict):

        ignore_ids = [TFields.TEMPLATE_ID, TFields.SOURCE, TFields.TARGET]
        return \
            {key: var_dict[key] for key in var_dict if key not in ignore_ids}



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from vitrage.common.constants import DatasourceProperties as DSProps
from vitrage.common.constants import EdgeLabel
from vitrage.common.constants import EntityCategory as Category
from vitrage.common.constants import VertexProperties as VProps
from vitrage.datasources.alarm_properties import AlarmProperties as AlarmProps
from vitrage.datasources.alarm_transformer_base import AlarmTransformerBase
from vitrage.datasources.collectd import COLLECTD_DATASOURCE
from vitrage.datasources.collectd.properties import\
    CollectdProperties as CProps
from vitrage.datasources import transformer_base as tbase
import vitrage.graph.utils as graph_utils
from vitrage.utils.datetime import format_unix_timestamp


class CollectdTransformer(AlarmTransformerBase):

    def _create_snapshot_entity_vertex(self, entity_event):
        # The Collectd datasource does not support snapshot mode
        return None

    def _create_update_entity_vertex(self, entity_event):
        return self._create_vertex(entity_event)

    def _create_vertex(self, entity_event):

        entity_event['timestamp'] = format_unix_timestamp(
            entity_event[CProps.TIME], tbase.TIMESTAMP_FORMAT)

        update_timestamp = entity_event['timestamp']

        vitrage_sample_timestamp = entity_event[DSProps.SAMPLE_DATE]

        entity_state = AlarmProps.INACTIVE_STATE \
            if self._ok_status(entity_event) else AlarmProps.ACTIVE_STATE

        metadata = {
            VProps.NAME: entity_event[CProps.MESSAGE],
            VProps.SEVERITY: entity_event[CProps.SEVERITY],
            VProps.RAWTEXT: self.generate_raw_text(entity_event),
            VProps.RESOURCE_NAME: entity_event[CProps.RESOURCE_NAME]
        }

        return graph_utils.create_vertex(
            self._create_entity_key(entity_event),
            vitrage_category=Category.ALARM,
            vitrage_type=entity_event[DSProps.ENTITY_TYPE],
            vitrage_sample_timestamp=vitrage_sample_timestamp,
            entity_state=entity_state,
            update_timestamp=update_timestamp,
            metadata=metadata)

    def _create_snapshot_neighbors(self, entity_event):
        return self._create_collectd_neighbors(entity_event)

    def _create_update_neighbors(self, entity_event):
        return self._create_collectd_neighbors(entity_event)

    def _create_collectd_neighbors(self, entity_event):
        graph_neighbors = entity_event.get(self.QUERY_RESULT, [])

        return [self._create_neighbor(entity_event,
                                      graph_neighbor[VProps.ID],
                                      graph_neighbor[VProps.VITRAGE_TYPE],
                                      EdgeLabel.ON,
                                      neighbor_category=Category.RESOURCE)
                for graph_neighbor in graph_neighbors]

    def _ok_status(self, entity_event):
        return entity_event[CProps.SEVERITY] == 'OK'

    def _create_entity_key(self, entity_event):

        entity_type = entity_event[DSProps.ENTITY_TYPE]
        alarm_id = entity_event[CProps.ID]
        resource_name = entity_event[CProps.RESOURCE_NAME]
        return tbase.build_key(self._key_values(entity_type,
                                                resource_name,
                                                alarm_id))

    def get_vitrage_type(self):
        return COLLECTD_DATASOURCE

    @staticmethod
    def generate_raw_text(entity_event):
        resources = [entity_event.get(CProps.TYPE_INSTANCE),
                     entity_event[CProps.PLUGIN],
                     entity_event.get(CProps.PLUGIN_INSTANCE)]
        return '-'.join([resource for resource in resources if resource])

    @staticmethod
    def get_enrich_query(event):
        resource_type = event.get(CProps.RESOURCE_TYPE)
        resource_name = event.get(CProps.RESOURCE_NAME)

        if resource_type and resource_name:
            return {VProps.NAME: resource_name,
                    VProps.VITRAGE_TYPE: resource_type}

        return None



# Copyright 2018 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from vitrage.common.constants import TemplateTypes
from vitrage.evaluator.template_fields import TemplateFields
from vitrage.evaluator.template_validation.content.v1.metadata_validator \
    import MetadataValidator
from vitrage.tests.unit.evaluator.template_validation.content.base import \
    ValidatorTest


class MetadataValidatorTest(ValidatorTest):

    def test_validate_metadata_standard(self):
        # In v1, all metadata validations are done by the syntax validator
        metadata = {TemplateFields.NAME: 'blabla',
                    TemplateFields.VERSION: '2',
                    TemplateFields.TYPE: TemplateTypes.STANDARD}
        result = MetadataValidator.validate(metadata)
        self._assert_correct_result(result)

    def test_validate_metadata_definition(self):
        # In v1, all metadata validations are done by the syntax validator
        metadata = {TemplateFields.NAME: 'blabla',
                    TemplateFields.VERSION: '2',
                    TemplateFields.TYPE: TemplateTypes.DEFINITION}
        result = MetadataValidator.validate(metadata)
        self._assert_correct_result(result)

    def test_validate_metadata_equivalence(self):
        # In v1, all metadata validations are done by the syntax validator
        metadata = {TemplateFields.NAME: 'blabla',
                    TemplateFields.VERSION: '2',
                    TemplateFields.TYPE: TemplateTypes.EQUIVALENCE}
        result = MetadataValidator.validate(metadata)
        self._assert_correct_result(result)

    def test_validate_metadata_invalid_type(self):
        # In v1, all metadata validations are done by the syntax validator
        metadata = {TemplateFields.NAME: 'blabla',
                    TemplateFields.VERSION: '2',
                    TemplateFields.TYPE: 'invalid type'}
        result = MetadataValidator.validate(metadata)
        self._assert_correct_result(result)

    def test_validate_metadata_no_type(self):
        # In v1, all metadata validations are done by the syntax validator
        metadata = {TemplateFields.NAME: 'blabla',
                    TemplateFields.VERSION: '2'}
        result = MetadataValidator.validate(metadata)
        self._assert_correct_result(result)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
import pecan

from oslo_log import log
from oslo_utils.strutils import bool_from_string
from pecan.core import abort

from vitrage.api.controllers.rest import RootRestController
from vitrage.api.policy import enforce


LOG = log.getLogger(__name__)


# noinspection PyBroadException
class AlarmCountsController(RootRestController):

    @pecan.expose('json')
    def index(self, all_tenants=False):
        return self.get(all_tenants)

    @pecan.expose('json')
    def get(self, all_tenants=False):
        all_tenants = bool_from_string(all_tenants)
        if all_tenants:
            enforce("get alarms count:all_tenants", pecan.request.headers,
                    pecan.request.enforcer, {})
        else:
            enforce("get alarms count", pecan.request.headers,
                    pecan.request.enforcer, {})

        LOG.info('received get alarm counts')

        try:
            alarm_counts_json = pecan.request.client.call(
                pecan.request.context, 'get_alarm_counts',
                all_tenants=all_tenants)

            return json.loads(alarm_counts_json)

        except Exception:
            LOG.exception('failed to get alarm count.')
            abort(404, 'Failed to get alarm count.')


class ResourceCountsController(RootRestController):

    @pecan.expose('json')
    def post(self, **kwargs):
        resource_type = kwargs.get('resource_type', None)
        all_tenants = kwargs.get('all_tenants', False)
        all_tenants = bool_from_string(all_tenants)
        query = kwargs.get('query')
        group_by = kwargs.get('group_by')
        if query:
            query = json.loads(query)

        if all_tenants:
            enforce("count resources:all_tenants", pecan.request.headers,
                    pecan.request.enforcer, {})
        else:
            enforce("count resources", pecan.request.headers,
                    pecan.request.enforcer, {})

        LOG.info('received get resource counts')

        try:
            resource_counts_json = pecan.request.client.call(
                pecan.request.context, 'count_resources',
                resource_type=resource_type,
                all_tenants=all_tenants,
                query=query,
                group_by=group_by)

            return json.loads(resource_counts_json)

        except Exception:
            LOG.exception('failed to get resource count.')
            abort(404, 'Failed to get resource count.')



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from vitrage.common.constants import VertexProperties as VProps


class PropsConverter(object):

    PROPS_CONVERSION = {
        'category': VProps.VITRAGE_CATEGORY,
        'type': VProps.VITRAGE_TYPE,
        'resource_id': VProps.VITRAGE_RESOURCE_ID,
        'sample_timestamp': VProps.VITRAGE_SAMPLE_TIMESTAMP,
        'is_deleted': VProps.VITRAGE_IS_DELETED,
        'is_placeholder': VProps.VITRAGE_IS_PLACEHOLDER,
        'aggregated_state': VProps.VITRAGE_AGGREGATED_STATE,
        'operational_state': VProps.VITRAGE_OPERATIONAL_STATE,
        'aggregated_severity': VProps.VITRAGE_AGGREGATED_SEVERITY,
        'operational_severity': VProps.VITRAGE_OPERATIONAL_SEVERITY
    }

    @classmethod
    def convert_props_with_set(cls, properties):
        converted_properties = set()
        for key, value in properties:
            new_key = cls.PROPS_CONVERSION[key] if key in \
                cls.PROPS_CONVERSION else key
            converted_properties.add((new_key, value))
        return converted_properties

    @classmethod
    def convert_props_with_dictionary(cls, properties):
        converted_properties = {}
        for key, value in properties.items():
            new_key = cls.PROPS_CONVERSION[key] if key in \
                cls.PROPS_CONVERSION else key
            converted_properties[new_key] = value
        return converted_properties



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from pysnmp.proto.rfc1902 import OctetString
from testtools import matchers

from vitrage.common.constants import VertexProperties as VProps
import vitrage.notifier.plugins.snmp.snmp_sender as sender
from vitrage.notifier.plugins.snmp.snmp_sender import SnmpSender
from vitrage.tests import base
from vitrage.tests.mocks import utils
from vitrage.tests.unit.notifier.snmp_notifier import common


class SnmpNotifierTest(base.BaseTest):
    def setUp(self):
        super(SnmpNotifierTest, self).setUp()

        self.cfg_fixture.config(
            group='snmp',
            alarm_oid_mapping=utils.get_resources_dir() +
            '/snmp_notifier/alarm_oid_mapping.yaml',
            consumers=utils.get_resources_dir() + '/snmp_notifier/dests.yaml',
            oid_tree=utils.get_resources_dir() +
            '/snmp_notifier/oid_tree_with_severity_mapping.yaml'
        )
        self.snmp_sender = SnmpSender()

    def test_create_oids(self):

        oids, var_lst = self.snmp_sender._build_oids()

        self.assertThat(oids, matchers.HasLength(4))
        self.assertThat(var_lst, matchers.HasLength(3))

        self.assertIn(VProps.NAME, oids)
        self.assertIn(VProps.VITRAGE_IS_DELETED, oids)
        self.assertIn(VProps.VITRAGE_OPERATIONAL_SEVERITY, oids)
        self.assertIn(sender.SEVERITY, oids)

        self.assertIn(VProps.NAME, var_lst)
        self.assertIn(VProps.VITRAGE_IS_DELETED, var_lst)
        self.assertIn(VProps.VITRAGE_OPERATIONAL_SEVERITY, var_lst)

    def test_var_binds(self):

        oid_with_alarm_objects = \
            common.GENERAL_OID + '.' + \
            common.COMPANY_OID + '.' + common.ALARM_OBJECTS_OID

        var_binds = self.snmp_sender._get_var_binds(common.alarm_data)

        self.assertThat(var_binds, matchers.HasLength(3))

        self.assertIn((oid_with_alarm_objects + '.' + common.NAME_OID,
                      OctetString(common.alarm_data.get(VProps.NAME,
                                                        sender.NA))),
                      var_binds)
        self.assertIn((oid_with_alarm_objects + '.' + common.IS_DELETED_OID,
                       OctetString(common.alarm_data.get
                                   (VProps.VITRAGE_IS_DELETED, sender.NA))),
                      var_binds)
        self.assertIn((oid_with_alarm_objects + '.' + common.SEVERITY_OID,
                       OctetString(common.alarm_data.get
                                   (VProps.VITRAGE_OPERATIONAL_SEVERITY,
                                    sender.NA))),
                      var_binds)

    def test_get_severity_oid(self):

        alert_severity_oid = \
            self.snmp_sender._get_severity_oid(common.alarm_data)

        self.assertEqual(alert_severity_oid, common.SEVERITY_OID)

    def test_get_alert_oid(self):

        alert_severity_oid = \
            self.snmp_sender._get_severity_oid(common.alarm_data)
        alert_details = self.snmp_sender.alarm_mapping.get(common.name_)

        # TODO(annarez): check if I need this assert
        self.assertEqual(alert_details.get(sender.OID), common.ALERT_OID)

        alert_oid = self.snmp_sender._get_alert_oid(alert_details[sender.OID],
                                                    alert_severity_oid)

        self.assertEqual(alert_oid, common.GENERAL_OID + '.' +
                         common.COMPANY_OID + '.' + common.ALARM_PREFIX_OID +
                         common.ALERT_OID + '.' + common.SEVERITY_OID)

    def test_get_details(self):

        alert_details, alert_severity_oid = \
            self.snmp_sender._get_details(common.alarm_data)

        self.assertEqual(alert_details, common.alert_details)
        self.assertEqual(alert_severity_oid, common.SEVERITY_OID)



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import os
import sys
import yaml

from vitrage.common.constants import TemplateStatus
from vitrage.common.constants import TemplateTypes as TType
from vitrage.evaluator.template_db.template_repository import \
    add_templates_to_db
from vitrage import storage
from vitrage.storage.sqlalchemy import models


class TestConfiguration(object):

    def add_db(self):
        db_name = "sqlite:///test-%s-%s.db" % (type(self).__name__,
                                               sys.version_info[0])
        self.config(group='database', connection=db_name)
        self._db = storage.get_connection_from_config()
        engine = self._db._engine_facade.get_engine()
        models.Base.metadata.drop_all(engine)
        models.Base.metadata.create_all(engine)
        return self._db

    def add_templates(self, templates_dir, templates_type=TType.STANDARD):
        yamls = [t for t in TestConfiguration.load_yaml_files(templates_dir)]
        templates = add_templates_to_db(self._db, yamls, templates_type)
        for t in templates:
            if t.status == TemplateStatus.LOADING:
                self._db.templates.update(t.uuid, 'status',
                                          TemplateStatus.ACTIVE)
            if t.status == TemplateStatus.DELETING:
                self._db.templates.update(t.uuid, 'status',
                                          TemplateStatus.DELETED)
        return templates

    @staticmethod
    def load_yaml_files(path):
        if os.path.isdir(path):
            file_paths = [path + "/" + fn for fn in os.listdir(path)
                          if os.path.isfile(path + "/" + fn)]
        else:
            file_paths = [path]

        yamls = []
        for file_path in file_paths:
            try:
                yamls.append(TestConfiguration._load_yaml_file(file_path))
            except Exception:
                continue
        return yamls

    @staticmethod
    def _load_yaml_file(path):
        with open(path, 'r') as stream:
            return yaml.load(stream, Loader=yaml.BaseLoader)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan


class VersionsController(object):

    @staticmethod
    @pecan.expose('json')
    def index():
        return {
            'versions': [
                {
                    'status': 'CURRENT',
                    'links': [
                        {
                            'rel': 'self',
                            'href': pecan.request.application_url + '/v1/'
                        }
                    ],
                    'id': 'v1.0',
                    'updated': '2015-11-29'
                }
            ]
        }



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from vitrage.common.constants import EventProperties as EventProps
from vitrage.datasources.nova.host import NOVA_HOST_DATASOURCE


class DoctorProperties(object):
    HOST_DOWN = 'compute.host.down'
    HOST_TYPE = NOVA_HOST_DATASOURCE
    UPDATE_TIME = 'update_time'
    CUSTOM_EVENT_UP = 'vitrage.custom.event.up'
    CUSTOM_EVENT_DOWN = 'vitrage.custom.event.down'


class DoctorDetails(object):
    """The details appear inside a details section in the event """
    HOSTNAME = 'hostname'
    STATUS = 'status'
    SEVERITY = 'severity'


class DoctorStatus(object):
    DOWN = 'down'
    UP = 'up'


def get_detail(alarm, detail):
    return alarm[EventProps.DETAILS][detail] if \
        alarm and EventProps.DETAILS in alarm and \
        detail in alarm[EventProps.DETAILS] \
        else None



# Copyright 2017 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import abc

from vitrage.evaluator.actions.base import ActionType
from vitrage.evaluator.actions.recipes.execute_mistral import WORKFLOW
from vitrage.evaluator.template_fields import TemplateFields
from vitrage.tests.unit.evaluator.template_validation.content.base import \
    ActionValidatorTest
from vitrage.tests.unit.evaluator.template_validation.content.base import \
    DEFINITIONS_INDEX_MOCK


class BaseExecuteMistralValidatorTest(ActionValidatorTest):

    @abc.abstractmethod
    def _create_execute_mistral_action(self, workflow, host, host_state):
        pass

    def _validate_execute_mistral_action(self, validator):
        self._validate_action(
            self._create_execute_mistral_action('wf_1', 'host_2', 'down'),
            validator.validate
        )

    def _validate_execute_mistral_action_without_workflow(self, validator):

        # Test setup
        idx = DEFINITIONS_INDEX_MOCK.copy()
        action = self._create_execute_mistral_action('wf_1', 'host_2', 'down')
        action[TemplateFields.PROPERTIES].pop(WORKFLOW)

        # Test action
        result = validator.validate(action, idx)

        # Test assertions
        self._assert_fault_result(result, 133)

    def _validate_execute_mistral_action_with_empty_workflow(self, validator):

        # Test setup
        idx = DEFINITIONS_INDEX_MOCK.copy()
        action = self._create_execute_mistral_action('', 'host_2', 'down')

        # Test action
        result = validator.validate(action, idx)

        # Test assertions
        self._assert_fault_result(result, 133)

    def _validate_execute_mistral_action_with_none_workflow(self, validator):

        # Test setup
        idx = DEFINITIONS_INDEX_MOCK.copy()
        action = self._create_execute_mistral_action(None, 'host_2', 'down')

        # Test action
        result = validator.validate(action, idx)

        # Test assertions
        self._assert_fault_result(result, 133)

    def _validate_execute_mistral_action_without_additional_props(self,
                                                                  validator):

        # Test setup - having only the 'workflow' param is a legal config
        idx = DEFINITIONS_INDEX_MOCK.copy()
        action = self._create_no_input_mistral_action('wf_1')

        # Test action
        result = validator.validate(action, idx)

        # Test assertions
        self._assert_correct_result(result)

    @staticmethod
    def _create_no_input_mistral_action(workflow):

        properties = {
            WORKFLOW: workflow,
        }
        action = {
            TemplateFields.ACTION_TYPE: ActionType.EXECUTE_MISTRAL,
            TemplateFields.PROPERTIES: properties
        }

        return action

    @staticmethod
    def _create_v1_execute_mistral_action(workflow, host, host_state,
                                          **kwargs):

        properties = {
            WORKFLOW: workflow,
            'host': host,
            'host_state': host_state
        }
        properties.update(kwargs)

        action = {
            TemplateFields.ACTION_TYPE: ActionType.EXECUTE_MISTRAL,
            TemplateFields.PROPERTIES: properties
        }

        return action

    @staticmethod
    def _create_v2_execute_mistral_action(workflow, host, host_state,
                                          **kwargs):

        input_props = {
            'host': host,
            'host_state': host_state
        }
        input_props.update(kwargs)
        properties = {
            WORKFLOW: workflow,
            'input': input_props
        }

        action = {
            TemplateFields.ACTION_TYPE: ActionType.EXECUTE_MISTRAL,
            TemplateFields.PROPERTIES: properties
        }

        return action



# Copyright 2016 - Alcatel-Lucent
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from networkx import is_frozen
from oslo_log import log as logging

from vitrage.common.constants import EdgeProperties as EProps
from vitrage.graph.algo_driver.algorithm import GraphAlgorithm
from vitrage.graph.algo_driver.algorithm import Mapping
from vitrage.graph.algo_driver.sub_graph_matching import NEG_CONDITION
from vitrage.graph.algo_driver.sub_graph_matching import subgraph_matching
from vitrage.graph.driver import Direction
from vitrage.graph.filter import check_filter
from vitrage.graph.query import create_predicate

LOG = logging.getLogger(__name__)


class NXAlgorithm(GraphAlgorithm):

    def __init__(self, graph):
        """Create a new GraphAlgorithm

        :param graph: graph instance
        :type graph: driver.Graph
        """
        super(NXAlgorithm, self).__init__(graph)

    @classmethod
    def _create_new_graph(cls, *args, **kwargs):
        from vitrage.graph.driver.networkx_graph import NXGraph
        return NXGraph(args, **kwargs)

    def graph_query_vertices(self,
                             root_id,
                             query_dict=None,
                             depth=None,
                             direction=Direction.BOTH,
                             edge_query_dict=None):
        graph = self._create_new_graph('graph')

        root_data = self.graph._g.nodes[root_id]

        match_func = create_predicate(query_dict) if query_dict else None
        edge_match_func = create_predicate(edge_query_dict) \
            if edge_query_dict else None

        if match_func and not match_func(root_data):
            LOG.info('graph_query_vertices: root %s does not match filter %s',
                     root_id, query_dict)
            return graph

        n_result = []
        visited_nodes = set()
        n_result.append((root_id, self.graph.get_vertex(root_id).properties))
        e_result = []
        nodes_q = [(root_id, 0)]
        while nodes_q:
            node_id, curr_depth = nodes_q.pop(0)
            if (node_id in visited_nodes) or (depth and curr_depth >= depth):
                continue
            visited_nodes.add(node_id)
            (n_list, e_list) = self.graph._neighboring_nodes_edges_query(
                node_id,
                direction=direction,
                vertex_predicate=match_func,
                edge_predicate=edge_match_func)
            n_result.extend(n_list)
            e_result.extend(e_list)
            nodes_q.extend([(v_id, curr_depth + 1) for v_id, data in n_list])

        graph = self._create_new_graph(graph.name)
        for v_id, data in n_result:
            graph._g.add_node(v_id, **data)
        for source_id, target_id, label, data in e_result:
            graph._g.add_edge(source_id, target_id, label, **data)

        return graph

    def sub_graph_matching(self,
                           subgraph,
                           known_match,
                           validate=False):
        """Finds all the matching subgraphs in the graph

        In case the known_match has a subgraph edge with property
        "negative_condition" then run subgraph matching on the edge vertices
        and unite the results.
        Otherwise just run subgraph matching and return its result.

        :param subgraph: the subgraph to match
        :param known_match: starting point at the subgraph and the graph
        :param validate:
        :return: all the matching subgraphs in the graph
        """
        sge = known_match.subgraph_element
        ge = known_match.graph_element

        if not known_match.is_vertex and sge.get(NEG_CONDITION):
            source_matches = self._filtered_subgraph_matching(ge.source_id,
                                                              sge.source_id,
                                                              subgraph,
                                                              validate)
            target_matches = self._filtered_subgraph_matching(ge.target_id,
                                                              sge.target_id,
                                                              subgraph,
                                                              validate)

            return self._list_union(source_matches, target_matches)
        else:
            return subgraph_matching(self.graph,
                                     subgraph,
                                     [known_match],
                                     validate)

    def create_graph_from_matching_vertices(self,
                                            query_dict=None,
                                            edge_attr_filter=None):
        vertices_ids = self.graph.get_vertices_ids(query_dict=query_dict)
        graph = self._create_new_graph('graph')
        graph._g = self.graph._g.subgraph(vertices_ids)

        # delete non matching edges
        if edge_attr_filter:
            self.apply_edge_attr_filter(graph, edge_attr_filter)

        return graph

    def subgraph(self, entities):
        subgraph = self._create_new_graph('graph')
        subgraph._g = self.graph._g.subgraph(entities)
        return subgraph

    def _filtered_subgraph_matching(self,
                                    ge_v_id,
                                    sge_v_id,
                                    subgraph,
                                    validate):
        """Runs subgraph_matching on edges vertices with filtering

        Runs subgraph_matching on edges vertices after checking if that vertex
        has real neighbors in the entity graph.
        """
        if self.graph.neighbors(ge_v_id,
                                edge_attr_filter={EProps.VITRAGE_IS_DELETED:
                                                  False}):
            template_vertex = subgraph.get_vertex(sge_v_id)
            graph_vertex = self.graph.get_vertex(ge_v_id)
            match = Mapping(template_vertex, graph_vertex, True)
            return subgraph_matching(self.graph, subgraph, [match], validate)

        return []

    @staticmethod
    def _list_union(list_1, list_2):
        """Union of list that aren't hashable

        Can't use here set union because the items in the lists are
        dictionaries and they are not hashable for set.

        :return: list - union list
        """

        for target_item in list_2:
            if target_item not in list_1:
                list_1.append(target_item)

        return list_1

    @staticmethod
    def apply_edge_attr_filter(graph, edge_attr_filter):
        edges = graph._g.edges(data=True, keys=True)
        edges_to_remove = [(u, v, k) for (u, v, k, d) in edges
                           if not check_filter(d, edge_attr_filter)]

        if edges_to_remove and is_frozen(graph._g):
            graph._g = graph._g.copy()

        for source, target, key in edges_to_remove:
            graph._g.remove_edge(u=source, v=target, key=key)



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class NagiosProperties(object):
    NUM_COLUMNS = 7
    RESOURCE_TYPE = 'resource_type'
    RESOURCE_NAME = 'resource_name'
    SERVICE = 'service'
    STATUS = 'status'
    LAST_CHECK = 'last_check'
    DURATION = 'duration'
    ATTEMPT = 'attempt'
    STATUS_INFO = 'status_info'
    NAGIOS = 'nagios'


class NagiosTestStatus(object):
    OK = 'OK'
    WARNING = 'WARNING'
    CRITICAL = 'CRITICAL'
    UNKNOWN = 'UNKNOWN'



# Copyright 2016 - Nokia
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import oslo_messaging
import pecan
from pecan import rest


class RootRestController(rest.RestController):

    @pecan.expose()
    def _route(self, args, request=None):
        """All requests go through here

        We can check the backend status
        """

        if not pecan.request.check_backend:
            return super(RootRestController, self)._route(args, request)

        try:
            client = pecan.request.client.prepare(timeout=5)
            backend_is_alive = client.call(pecan.request.context, 'is_alive')
            if backend_is_alive:
                return super(RootRestController, self)._route(args, request)
            else:
                pecan.abort(503, detail='vitrage-graph is not ready')
        except oslo_messaging.MessagingTimeout:
            pecan.abort(503, detail='vitrage-graph not available')



# Copyright 2019 - Viettel
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg

from vitrage.common.constants import DatasourceOpts as DSOpts
from vitrage.datasources.kapacitor.config import KapacitorConfig
from vitrage.datasources.kapacitor import KAPACITOR_DATASOURCE
from vitrage.datasources.nova.host import NOVA_HOST_DATASOURCE
from vitrage.datasources.nova.instance import NOVA_INSTANCE_DATASOURCE
from vitrage.tests import base
from vitrage.tests.mocks import utils


class TestKapacitorConfig(base.BaseTest):

    OPTS = [
        cfg.StrOpt(DSOpts.TRANSFORMER,
                   default='vitrage.datasources.kapacitor.transformer.'
                           'KapacitorTransformer',
                   help='Kapacitor data source transformer class path',
                   required=True),
        cfg.StrOpt(DSOpts.DRIVER,
                   default='vitrage.datasources.kapacitor.driver.'
                           'KapacitorDriver',
                   help='Kapacitor driver class path',
                   required=True),
        cfg.StrOpt(DSOpts.CONFIG_FILE,
                   help='Kapacitor configuration file',
                   default=utils.get_resources_dir()
                        + '/kapacitor/kapacitor_conf.yaml'),
    ]

    def setUp(self):
        super(TestKapacitorConfig, self).setUp()
        self.conf_reregister_opts(self.OPTS, group=KAPACITOR_DATASOURCE)

    def test_get_vitrage_resource(self):
        """Test the resource returned after processing a list of mappings

        :return:
        """
        # Action
        kapacitor_conf = KapacitorConfig()

        # Test assertions
        mapped_resource = kapacitor_conf.get_vitrage_resource(None)
        self.assertIsNone(mapped_resource, 'expected None')

        mapped_resource = kapacitor_conf.get_vitrage_resource('')
        self.assertIsNone(mapped_resource, 'expected None')

        mapped_resource = kapacitor_conf.get_vitrage_resource('cloud.compute1')
        self.assertIsNotNone(mapped_resource, 'expected Not None')
        self.assertEqual(NOVA_HOST_DATASOURCE, mapped_resource[0])
        self.assertEqual('compute-1', mapped_resource[1])

        mapped_resource = kapacitor_conf.get_vitrage_resource('compute-2')
        self.assertIsNotNone(mapped_resource, 'expected Not None')
        self.assertEqual(NOVA_HOST_DATASOURCE, mapped_resource[0])
        self.assertEqual('compute-2', mapped_resource[1])

        mapped_resource = kapacitor_conf.get_vitrage_resource('instance-1')
        self.assertIsNotNone(mapped_resource, 'expected Not None')
        self.assertEqual(NOVA_INSTANCE_DATASOURCE, mapped_resource[0])
        self.assertEqual('instance-1', mapped_resource[1])

    @staticmethod
    def _assert_equals(mapping1, mapping2):
        return mapping1.kapacitor_host_regexp.pattern == \
            mapping2.kapacitor_host_regexp.pattern and \
            mapping1.resource_type == mapping2.resource_type and \
            mapping1.resource_name == mapping2.resource_name
